/**
 * Created by Administrator on 2016/1/15.
 */

var GameAnimation = cc.Sprite.extend({

    // 当前播放的帧
    frameIndex:null,
    // 当前最大帧数
    frameMaxIndex:null,
    // 帧的时间
    lastFrameTime:null,
    // 帧的间隔
    frameIntervalTime:null,
    // 图片路径数组
    imgeUrls:null,
    // 是否初始化
    isInit: null,
    //播放次数
    playCount:null,
    //当前播放次数
    curPlayCount:null,
    // 是否停止
    isStop:null,
    // 播放完成是否删除
    isRemoveFromComplete:null,
    // 每帧回调方法
    frameFun:null,
    // 帧播放完成函数
    frameCompleteFun:null,
    // 回调信息
    selectorTargetInfo:null,
    // 文件夹 纹理前缀
    folderName:null,
    // 播到第几帧
    playFrameIndex:null,
    // 从第几帧开始播
    fromFrameIndex:null,
    // 是否缓冲
    isPool:false,

    onEnter:function ()
    {
        this._super();
    },

    ctor: function ()
    {
        this._super();
        this.folderName = "";
        this.resetAttribute();
    },

    resetAttribute:function()
    {
        this.isPool = false;
        this.frameIndex = 0;
        this.frameMaxIndex = 0;
        this.lastFrameTime = 0;
        this.frameIntervalTime = 0;
        this.imgeUrls = [];
        this.isInit = false;
        this.playCount = -1;
        this.curPlayCount = 0;
        this.isStop = false;
        this.isRemoveFromComplete = false;
        this.frameFun = null;
        this.frameCompleteFun = null;
        this.selectorTargetInfo = null;
    },

    reuse:function()
    {
        this.resetAttribute();
    },

    initialize:function(data)
    {
        if (this.isInit == true)
        {
            return;
        }

        this.setInfo(data);
        this.scheduleUpdate();
    },

    update:function()
    {
        this.playFrame();
    },

    playFrame:function()
    {
        if(this.isStop == true)
        {
            return;
        }

        if(engineGameRunTime() - this.lastFrameTime > this.frameIntervalTime)
        {
            if(this.frameIndex > this.playFrameIndex)
            {
                this.curPlayCount++;
                if(this.playCount == -1)
                {
                    this.frameIndex = this.fromFrameIndex;
                }
                else
                {
                    if(this.curPlayCount < this.playCount)
                    {
                        this.frameIndex = this.fromFrameIndex;
                    }
                    else
                    {
                        // 回调完成函数
                        this.gotoAndStop(this.playFrameIndex);

                        if(this.frameCompleteFun != null)
                        {
                            this.frameCompleteFun();
                        }
                        if(this.isRemoveFromComplete == true)//播放完成之后移除
                        {
                            this.dispose();
                        }
                        return;
                    }
                }
            }

            this.lastFrameTime = engineGameRunTime();
            this.setSpriteFrame(this.folderName + "/" +  this.imgeUrls[this.frameIndex] + ".png");

            if(this.frameFun != null)
            {
                this.frameFun(this.frameIndex, this.selectorTargetInfo);
            }

            this.frameIndex++;
        }
    },

    //设置帧率
    setFrameIntervalTime:function(fps)
    {
        this.frameIntervalTime = parseInt((1 / fps)*1000);
    },

    //开始播放
    gotoAndPlay:function(frameIndex)
    {
        this.isStop = false;
        this.lastFrameTime = 0;
        this.frameIndex = frameIndex;
        this.setSpriteFrame(this.folderName + "/" +  this.imgeUrls[this.frameIndex] + ".png");
    },

    //从第几帧播到第几帧
    fromFrameIndexToPlayFrameIndex:function(fromFrameIndex, playFrameIndex)
    {
        this.isStop = false;
        this.fromFrameIndex = fromFrameIndex;
        this.frameIndex = fromFrameIndex;
        this.lastFrameTime = 0;
        this.setSpriteFrame(this.folderName + "/" +  this.imgeUrls[this.frameIndex] + ".png");
        this.playFrameIndex = playFrameIndex;
    },

    //停止到某一帧
    gotoAndStop:function(frameIndex)
    {
        this.isStop = true;
        this.lastFrameTime = 0;
        this.frameIndex = frameIndex;
        this.setSpriteFrame(this.folderName + "/" + this.imgeUrls[this.frameIndex] + ".png");
    },

    setIsStop:function(bool)
    {
        this.isStop = bool;
    },

    // 重置信息
    resetInfo:function(data)
    {
        this.setInfo(data);
    },

    // 设置渲染信息 内部方法
    setInfo:function(data)
    {
        this.folderName = data.folderName;
        this.curPlayCount = 0;
        this.frameIndex = 0;
        this.lastFrameTime = 0;
        this.fromFrameIndex = 0;
        this.frameMaxIndex = data.frameMaxIndex;
        this.playFrameIndex = this.frameMaxIndex - 1;
        if(data.isRemoveFromComplete != null)
        {
            this.isRemoveFromComplete = data.isRemoveFromComplete;
        }
        if(data.playCount != null)
        {
            this.playCount = data.playCount;
        }
        this.imgeUrls = data.framesBitmapList;
        this.gotoAndPlay(this.frameIndex);
    },

    dispose:function()
    {
        // 如果是缓存 则需要还给引擎
        if(this.isPool)
        {
            engine.gamePool.putInPool(this);
        }
        this.removeFromParent();
    }
})

var createGameAnimation = function(jsonName, mcName, FPS, folderName, isPool)
{
    var data = skeletalAnimation[jsonName + "|" + mcName];
    if(data == null)
    {
        multipleAnimation(jsonName, jsonName);
        data = skeletalAnimation[jsonName + "|" + mcName];
    }
    var gameAnimation;
    if(isPool == true)
    {
        gameAnimation = engine.gamePool.popOut(GameAnimation);
    }
    else
    {
        gameAnimation = new GameAnimation();
    }
    data.folderName = folderName;
    gameAnimation.initialize(data);
    //设置帧率
    gameAnimation.setFrameIntervalTime(FPS);
    return gameAnimation;
}