// 骨骼动画数据类
var skeletalAnimation = new Object();

var GameSkeletalAnimation = cc.Node.extend({

    // 当前播放的帧
    frameIndex:null,
    // 帧的时间
    lastFrameTime:null,
    // 帧的间隔（需外部设置）
    frameIntervalTime:null,
    // 是否初始化
    isInit: null,
    // 帧数据
    framesData:null,
    // 帧的图
    framesBitmapUrlList:null,
    //播放次数
    playCount:null,
    //播放完成是否删除
    isRemoveFromComplete:null,
    //当前播放次数
    curPlayCount:null,
    //最大帧数
    frameMaxIndex:null,
    // 播到第几帧
    playFrameIndex:null,
    // 从第几帧开始播
    fromFrameIndex:null,
    //是否停止
    isStop:null,
    // 回调函数
    frameCompleteFun:null,
    // 是否缓冲
    isPool:false,
    // 每帧回调方法
    frameFun:null,
    // json文件
    jsonName:null,
    // 文件夹 纹理前缀
    folderName:null,
    onEnter: function ()
    {
        this._super();
        this.scheduleUpdate();
    },

    ctor: function ()
    {
        this._super();
        this.frameIndex = 0;
        this.lastFrameTime = 0;
        this.playCount = -1;
        this.curPlayCount = 0;
        this.frameMaxIndex = 0;
        this.playFrameIndex = 1;
        this.frameBitmapList = [];
        this.isInit = false;
        this.isStop = false;
        this.isRemoveFromComplete = false;
        this.folderName = "";
        this.isPool = false;
    },

    initialize:function(data)
    {
        if (this.isInit == true)
        {
            return;
        }

        this.setInfo(data);
    },

    update:function()
    {
        if(this.isStop == true)
        {
            return;
        }

        if(engineGameRunTime() - this.lastFrameTime > this.frameIntervalTime)
        {
            this.lastFrameTime = engineGameRunTime();

            if(this.frameIndex > this.playFrameIndex)
            {
                this.curPlayCount++;
                if(this.playCount == -1)
                {
                    this.frameIndex = this.fromFrameIndex;
                }
                else
                {
                    if(this.curPlayCount < this.playCount)
                    {
                        this.frameIndex = this.fromFrameIndex;
                    }
                    else
                    {
                        // 回调完成函数
                        this.gotoAndStop(this.playFrameIndex);

                        if(this.frameCompleteFun != null)
                        {
                            this.frameCompleteFun();
                        }
                        if(this.isRemoveFromComplete == true)//播放完成之后移除
                        {
                            this.removeFromParent();
                        }
                        return;
                    }
                }
            }

            for(var n = 0; n < this.frameBitmapList.length; n++)
            {
               this.frameBitmapList[n].setBitmapInfo(this.frameIndex);
                // 渐变消失时候有用到
                if(this.opacity != 255)
                {
                    this.frameBitmapList[n].opacity = this.opacity;
                }
            }

            if(this.frameFun != null)
            {
                this.frameFun(this.frameIndex);
            }

            this.frameIndex++;
        }
    },

    dispose:function()
    {
        // 如果是缓存 则需要还给引擎
        for(var n = 0; n < this.frameBitmapList.length; n++)
        {
            engine.gamePool.putInPool(this.frameBitmapList[n]);
        }
        this.removeFromParent();
    },

    // 设置播放完成回调
    steFrameComplete:function(callFunc)
    {
        this.frameCompleteFun = callFunc;
    },

    //设置帧率
    setFrameIntervalTime:function(fps)
    {
        this.frameIntervalTime = parseInt((1 / fps)*1000);
    },

    //开始播放
    gotoAndPlay:function(frameIndex)
    {
        this.isStop = false;
        this.frameIndex = frameIndex;
        for(var n = 0; n < this.frameBitmapList.length; n++)
        {
            var btimap = this.frameBitmapList[n];
            btimap.setBitmapInfo(this.frameIndex);
        }
    },

    //停止到某一帧
    gotoAndStop:function(frameIndex)
    {
        this.isStop = true;
        this.frameIndex = frameIndex;
        for(var n = 0; n < this.frameBitmapList.length; n++)
        {
            var btimap = this.frameBitmapList[n];
            btimap.setBitmapInfo(this.frameIndex);
        }
    },

    //从第几帧播到第几帧
    fromFrameIndexToPlayFrameIndex:function(fromFrameIndex, playFrameIndex)
    {
        this.isStop = false;
        this.fromFrameIndex = fromFrameIndex;
        this.frameIndex = fromFrameIndex;
        for(var n = 0; n < this.frameBitmapList.length; n++)
        {
            var btimap = this.frameBitmapList[n];
            btimap.setBitmapInfo(this.frameIndex);
        }
        this.playFrameIndex = playFrameIndex;
    },

    setIsStop:function(bool)
    {
        this.isStop = bool;
    },

    // 重置信息
    resetInfo:function(data)
    {
        this.setInfo(data);
    },

    // 设置渲染信息 内部方法
    setInfo:function(data)
    {
        var arr = this.getChildren();
        for(var i = arr.length - 1; i >= 0; i--)
        {
            arr[i].removeFromParent();
        }

        this.jsonName = data.jsonName;
        this.folderName = data.folderName;

        this.fromFrameIndex = 0;
        this.frameIndex = 0;
        this.curPlayCount = 0;
        this.frameBitmapList = [];
        if(data.isRemoveFromComplete != null)
        {
            this.isRemoveFromComplete = data.isRemoveFromComplete;
        }
        if(data.playCount != null)
        {
            this.playCount = data.playCount;
        }
        this.frameMaxIndex = data.frameMaxIndex;
        this.playFrameIndex = this.frameMaxIndex - 1;
        if(data.frameIntervalTime == null)
        {
            this.frameIntervalTime = parseInt(1 / (cc.game.config[cc.game.CONFIG_KEY.frameRate]) * 1000);
        }
        else
        {
            this.frameIntervalTime = data.frameIntervalTime;
        }
        this.framesBitmapList = data.framesBitmapList;
        this.framesData = data.framesBitmapInfo;

        for(var n = 0; n < this.framesBitmapList.length; n++)
        {
            var bitmapData = new Object();
            bitmapData.url = this.framesBitmapList[n].url;
            bitmapData.imgeData = this.framesData[this.framesBitmapList[n].url + "|" + n];
            bitmapData.px = this.framesBitmapList[n].px;
            bitmapData.py = this.framesBitmapList[n].py;
            bitmapData.jsonName = this.jsonName;
            bitmapData.folderName = this.folderName;
            bitmapData.isPool = this.isPool;
            var btimap = createGameBitmap(bitmapData);
            this.addChild(btimap, this.framesBitmapList[n].zIndex);
            btimap.setBitmapInfo(this.frameIndex);
            this.frameBitmapList.push(btimap);
        }
    },

    // 获取根据路径 获取图片
    getBitmapByUrl:function(bitmapUrl)
    {
        for(var n = 0; n < this.frameBitmapList.length; n++)
        {
            var bitmap = this.frameBitmapList[n];
            if(bitmap.url == bitmapUrl)
            {
                return bitmap;
            }
        }

        return null;
    },

    setBlendFunc:function(src, dst)
    {
        for(var n = 0; n < this.frameBitmapList.length; n++)
        {
            this.frameBitmapList[n].setBlendFunc(src, dst);
        }
    }
})

var createGameBitmap = function(data)
{
    var bitmap;
    // 如果这个是序列帧
    if(data.url.substr(0, 4) == "im::")
    {
        var imgName = (data.url.substr(4, data.url.length)).split(".");
        // 这里的帧率是由外部骨骼动画控制的传多少都无效
        bitmap = createGameAnimation(data.jsonName, imgName[0], 60, data.folderName, data.isPool);
        bitmap.setAnchorPoint(cc.p(data.px, data.py));
        bitmap.unscheduleUpdate();
        bitmap.imgeData = data.imgeData;
        bitmap.url = data.url;
        bitmap.setBitmapInfo = function(index)
        {
            var data = this.imgeData[index];

            this.setSpriteFrame(this.folderName + "/" + this.imgeUrls[index] + ".png");

            if(this.opacity != data.opacity)
            {
                this.setOpacity(data.opacity);
            }
            this.x = data.x;
            this.y = - data.y;

            if(this.skewY != data.skewY)
            {
                this.skewY = data.skewY;
            }
            if(this.skewX != data.skewX)
            {
                this.skewX = data.skewX;
            }

            if(this.scaleY != data.scaleY)
            {
                this.setScaleY(data.scaleY);
            }

            this.setRotation(data.rotation);
        }
    }
    else
    {
        var bitmap;
        if(data.isPool == true)
        {
            bitmap = engine.gamePool.popOut(cc.Sprite);
        }
        else
        {
            bitmap = new cc.Sprite();
        }

        bitmap.setAnchorPoint(cc.p(data.px, data.py));
        bitmap.setSpriteFrame(data.folderName + "/" + data.url);
        bitmap.url = data.url;
        bitmap.imgeData = data.imgeData;

        bitmap.setBitmapInfo = function(index)
        {
            var data = this.imgeData[index];
            if(this.opacity != data.opacity)
            {
                this.setOpacity(data.opacity);
            }
            this.x = data.x;
            this.y = - data.y;

            if(this.skewY != data.skewY)
            {
                this.skewY = data.skewY;
            }

            if(this.skewX != data.skewX)
            {
                this.skewX = data.skewX;
            }

            if(this.scaleX != data.scaleX)
            {
                this.setScaleX(data.scaleX);
            }

            if(this.scaleY != data.scaleY)
            {
                this.setScaleY(data.scaleY);
            }

            this.setRotation(data.rotation);
        }
    }

    return bitmap;
}

//遍历获取同一个json文件里的不同mc动画信息
var multipleAnimation = function (animationKey, animationJson)
{
    // 遍历MC中的所有MC
    var num = 0;
    var data = cc.loader.getRes(animationJson);
    while(data["mc" + num.toString()] != null || data["im" + num.toString()] != null)
    {
        var key = "";
        if(data["mc" + num.toString()] != null)
        {
            key = animationKey + "|" + data["mc" + num.toString()].name;
            analysisSkeletalAnimation(key, data["mc" + num.toString()]);
        }
        if(data["im" + num.toString()] != null)
        {
            key = animationKey + "|" + data["im" + num.toString()].name;
            analysisAnimation(key, data["im" + num.toString()]);
        }
        num++;
    }
}

var analysisAnimation = function (animationKey, animationObj)
{
    if(skeletalAnimation[animationKey] == null)
    {
        skeletalAnimation[animationKey] = new Object();
        var mcInfo = animationObj;
        skeletalAnimation[animationKey].frameMaxIndex = mcInfo.max;
        skeletalAnimation[animationKey].framesBitmapList = mcInfo.bi.split("|");
    }
}

//存储动画信息
//animationKey 当前资源路径键值
//animationObj 动画信息对象
//url 资源路径
var analysisSkeletalAnimation = function (animationKey, animationObj)
{
    if(skeletalAnimation[animationKey] == null)
    {
       var mcInfo = animationObj;
       skeletalAnimation[animationKey] = new Object();
       skeletalAnimation[animationKey].frameMaxIndex = mcInfo.max;
       skeletalAnimation[animationKey].framesBitmapList = [];
       skeletalAnimation[animationKey].framesBitmapInfo = new Object();

       if(mcInfo != null)
       {
           var datanum = 0;
           // 获取每个MC里面图片的信息
           while(mcInfo["mcdata" + datanum.toString()] != null)
           {
               var bitmapdata = mcInfo["mcdata" + datanum.toString()];
               var bitmapInfo = new Object();
               bitmapInfo.url = bitmapdata.url + ".png";
               bitmapInfo.zIndex = bitmapdata.zi;
               bitmapInfo.px  = parseFloat(bitmapdata.px);
               bitmapInfo.py  = parseFloat(bitmapdata.py);
               skeletalAnimation[animationKey].framesBitmapList.push(bitmapInfo);
               skeletalAnimation[animationKey].framesBitmapInfo[bitmapInfo.url + "|" + datanum] = [];
               var framearr = bitmapdata.bi.split("|");
               for(var h = 0; h < framearr.length; h++)
               {
                   var bitmapInfoArr = framearr[h].split(",");
                   var bitmapFrameInfo = new Object();
                   bitmapFrameInfo.x = parseFloat(bitmapInfoArr[0]);
                   bitmapFrameInfo.y = parseFloat(bitmapInfoArr[1]);
                   bitmapFrameInfo.scaleX = parseFloat(bitmapInfoArr[2]);
                   bitmapFrameInfo.scaleY = parseFloat(bitmapInfoArr[3]);
                   bitmapFrameInfo.opacity = parseFloat(bitmapInfoArr[4]) * 255;
                   bitmapFrameInfo.rotation = parseFloat(bitmapInfoArr[5]);
                   bitmapFrameInfo.skewX = parseFloat(bitmapInfoArr[6]);
                   bitmapFrameInfo.skewY = parseFloat(bitmapInfoArr[7]);
                   skeletalAnimation[animationKey].framesBitmapInfo[bitmapInfo.url + "|" + datanum].push(bitmapFrameInfo);
               }
               datanum++;
           }
        }
    }
}

var createGameSkeletalAnimation = function(jsonName, mcName, FPS, folderName, isPool)
{
    var data = skeletalAnimation[jsonName + "|" + mcName];
    if(data == null)
    {
        printLog(jsonName + "|" + mcName + "资源需要加载");
        multipleAnimation(jsonName, jsonName);
        data = skeletalAnimation[jsonName + "|" + mcName];
    }
    var mc = new GameSkeletalAnimation();
    if(isPool == true)
    {
        mc.isPool = true;
    }
    data.jsonName = jsonName;
    data.folderName = folderName;
    mc.initialize(data);
    //设置帧率
    mc.setFrameIntervalTime(FPS);
    return mc;
}