var MAP_TYPE = {
    H: 1, // 水平移动
    V: 2  // 垂直移动
};
var MapObstacle = cc.Node.extend({
    mainUI: null,
    type: MAP_TYPE.V, // 地图类型
    id: 1,
    num: 0,  // 第几个地图
    deltaY:0,  // 高度
    deltaX: 0,  // 移动宽度
    moveDistance: null, // 水平移动的最大距离
    isMoveEnd: false,
    golds: [],  // 生成金币
    floorTags:[], // 最开始的tag
    wallTags:[],
    ropes:[],     // 绳子的开始位置
    hooks:[],     // 钩子的初始位置
    whiteWalls:[],  // 白色底墙
    decorates: [],  // 装饰品
    clipper: null,
    isLoad: false, // 是否已经加载了
    obj:{},
    ctor: function(id, num){
        this._super();
        this.id = id || 1;
        if(this.id > 3000 && this.id < 4000){
            this.type = MAP_TYPE.H;
        }
        this.num = num || 0;
        this.initData();
    },
    initData: function(){
        this.hooks = [];
        this.golds = [];
        this.whiteWalls = [];
        this.decorates = [];
        this.ropes = [];
        this.hooks = [];
        this.clipper = null;
        this.moveDistance = null;
        this.deltaY = 0;
        this.deltaX = 0;
        this.floorTags = [];
        this.obj = {};
    },
    onEnter: function(){
        this._super();
        this.initUI();
        this.setMapType();
        this.getMaxMapWidth();
        this.createGold();
        this.registerEvent();
        this.hitRectForFloor();
        this.hitRectForWall();
        if(this.id > 4000){
            this.addWarningAction();
        }
        // this.initHook();
        // this.initText();
    },
    registerEvent: function(){
        // 眼睛消息
        Event.register(EVENT_ID.EYE, function(){
            if(this.id > 4000 || this.decorates.length <= 0 || this.clipper === null){ return;}
            this.clipper.stencil.setVisible(gameUserData.eye === 1);
            for(var i = 0;i < this.decorates.length; i += 1){
                if(this.decorates[i]){
                    this.decorates[i].setVisible(gameUserData.eye === 1);
                }
            }
        }, this)
    },
    onExit: function(){
        this._super();
        Event.remove(EVENT_ID.EYE, this);
    },
    initUI: function(){
        var json = Utils.getMapJsonByID(this.id);
        var nodeRes = cc.loader.getRes(json);
        var data = nodeRes.Content.Content.ObjectData;
        this.mainUI = Utils.parseNodeToUI(data, this.num);
        this.addChild(this.mainUI);
        this.createWhiteWall();
    },
    initText: function(){
        this.mainUI.text.setString(this.num * 10);
    },
    // 初始化钩子的位置
    initHook: function(){
        if(this.id < 4000){
            return ;
        }

        if(this.hooks.length <= 0){
            this.hooks[0] = this.mainUI.wallNode["wall_1"].getPosition();
            this.hooks[1] = this.mainUI.wallNode["wall_2"].getPosition();
            this.ropes[0] = this.mainUI.decorateNode["rope1"].getScaleY();
            this.ropes[1] = this.mainUI.decorateNode["rope2"].getScaleY();
        }


        this.mainUI.wallNode["wall_1"].setPosition(this.hooks[0]);
        this.mainUI.wallNode["wall_2"].setPosition(this.hooks[1]);
        this.mainUI.decorateNode["rope1"].setScaleY(this.ropes[0]);
        this.mainUI.decorateNode["rope2"].setScaleY(this.ropes[1]);

        this.mainUI.wallNode["wall_3"]["warningAction"].setVisible(true);
        this.mainUI.wallNode["wall_4"]["warningAction"].setVisible(true);
    },
    // 重置底板的tag
    refreshFloor: function(){
        var floors = this.getFloorsNode();
        var children = floors.getChildren();
        for(var i = 1; i <= children.length; i += 1){
            if(!this.floorTags[i-1]){
                this.floorTags[i-1] = floors["floor_" + i].getTag();
            }
            floors["floor_" + i].setTag(this.floorTags[i-1] + this.num * 10);
        }

        this.mainUI.text.setString((this.num + 1) * 10);
        this.isMoveEnd = false;
        this.initHook();
    },
    // 获取墙
    getWallsNode: function(){
        return this.mainUI.wallNode;
    },
    // 获取地板
    getFloorsNode: function(){
        return this.mainUI.floorNode;
    },
    // 获取最后一块地板
    getLastFloor: function(){
        var children = this.mainUI.floorNode.getChildren();
        return this.mainUI.floorNode["floor_" + children.length];
    },
    // 获取第一个快
    getFirstFloor: function(){
        return this.mainUI.floorNode["floor_" + 1];
    },
    // 获取地图类型
    getMapType: function(){
        return this.type;
    },
    // 设置类型
    setMapType: function(){
        var distance = this.getMapWidth();
        if(distance > 200){
            this.type = MAP_TYPE.H;
        }
    },
    // 获取地图跳跃高度
    getMapHeight: function(){
        // if(!this.deltaY){
        //     var node = this.getFloorsNode();
        //     var length = node.getChildren().length;
        //     this.deltaY = (node["floor_" + length].y - node["floor_1"].y)/9;
        // }
        return 100;
    },
    // 获取地图的横向偏移
    getMapWidth: function(){
        if(!this.deltaX){
            var node = this.getFloorsNode();
            var length = node.getChildren().length;
            var last = node["floor_" + length];
            this.deltaX = -(last.x - last.getContentSize().width/2)/9;
        }
        return this.deltaX;
    },
    // 获取地图的横向偏移
    getMaxMapWidth: function(){
        if(this.moveDistance === null){
            var node = this.getFloorsNode();
            var length = node.getChildren().length;
            var last = node["floor_" + length];
            this.moveDistance = -(last.x - last.getContentSize().width/2);
        }
    },
    // 地图的水平移动方向
    getMapDir: function(){
        if(this.getLastFloor().x > this.getFirstFloor().x){
            return ROLE_STATE.LEFT;
        } else {
            return ROLE_STATE.RIGHT;
        }
    },
    // 获取另一个滑轮
    getOtherPulley: function(target){
        var num = Utils.getNumberFromString(target.getName());
        if(num === 1){
            num = 2;
        }else if(num === 2){
            num = 1;
        }
        return target.getParent().getChildByName("wall_" + num);
    },
    // 绳子变化
    ropeChane: function(floor,y,time){
        var num = Utils.getNumberFromString(floor.getName());
        if(num > 2){ return ;}
        var scope = this.mainUI.decorateNode["rope"+num];
        var size = scope.getContentSize();
        var distance = scope.y - y;
        scope.runAction(cc.scaleTo(time, 1, distance/size.height));
    },
    // 关闭警告灯
    closeWarning: function(floor){
        var num = Utils.getNumberFromString(floor.getName());
        if(num < 3){ return;}
        this.mainUI.wallNode["wall_" + (num )]["warningAction"].setVisible(false);
    },
    // 生成金币
    createGold: function(){
        for(var i = 0; i < 8; i += 1){
            this.golds[i] = new cc.Sprite("#gold1.png");
            this.golds[i].setVisible(false);
            var size = this.golds[i].getContentSize();
            this.golds[i]["gameRect"] = cc.rect(0, 0, size.width, size.height);
            this.addChild(this.golds[i], 1001);
        }
        return this.golds[i];
    },
    // 恢复装饰
    recoveryDecorate: function(){
        for(var i = 0; i < this.whiteWalls.length; i += 1){
            this.whiteWalls[i].setVisible(true);
        }

        for(var j = 0; j < this.decorates.length; j += 1){
            this.decorates[j].setVisible(gameUserData.eye === 1);
        }

        var children = this.getFloorsNode().getChildren();
        for(var k = 0; k < children.length; k += 1){
            children[k].setVisible(true);
        }

        var walls = this.getWallsNode().getChildren();
        for(var num = 0; num < walls.length; num += 1){
            walls[num].setVisible(true);
        }
    },
    // 获取楼层数量  禁止同层出现2个以上的金币
    getFloorNum: function () {
        var length = this.mainUI.floorNode.getChildren().length;
        var num = Utils.getRandomNum(length-1) + 1;
        if(this.obj["floor" + num] && this.obj["floor"+ num] >= 2){
            return this.getFloorNum();
        }else{
            this.obj["floor"+ num] += 1;
            // cc.log("num = " + num);
            return num;
        }
    },
    // 随机生成金币的位置
    randGoldPosition: function(value){
        this.obj = {};
        for(var i = 0; i < value; i += 1){
            if(!this.golds[i]){ return;}
            var length = this.mainUI.floorNode.getChildren().length;
            // var num = Utils.getRandomNum(length-1) + 1;
            var num = this.getFloorNum();
            var floor = this.mainUI.floorNode.getChildByName("floor_" + num);
            if(!floor){
                return
            }
            var width = floor.getContentSize().width * floor.getScaleX();
            var x = floor.x - width/2 + Utils.getRandomNum(width - 80) + 40;
            if(!this.isOverlap(x)){
                this.golds[i].setPosition(cc.p(x, floor.y + 40));
                this.golds[i].setVisible(true);
                this.golds[i].stopAllActions();
                this.golds[i].runAction(AnimationMgr.getGoldRotateAction(1/10).repeatForever());
            }
        }
    },
    // 判断金币是否重叠
    isOverlap: function(x){
        for(var i = 0; i < this.golds.length; i += 1){
            if(Math.abs(this.golds[i].x - x) < this.golds[i].getContentSize().width - 10){
                return true;
            }
        }
        return false;
    },
    // 创建白色墙
    createWhiteWall: function(){
        if(this.id > 4000){ // 钢架
            return;
        }
        var children = this.getFloorsNode().getChildren();
        var clipper = new cc.ClippingNode();
        clipper.setInverted(true);
        clipper.stencil = new cc.Node();
        clipper.stencil.setVisible(gameUserData.eye === 1);
        for(var i = 0; i < children.length; i += 1){
            var child = children[i];
            var node = this.createBgNode(child);
            this.whiteWalls[i] = node;

            child.whiteWall = node;
            this.createDecorate(clipper, child);
            clipper.addChild(this.whiteWalls[i], -1);
        }
        this.addChild(clipper, -2);
        this.clipper = clipper;
    },
    // 创建白色底图
    createBgNode: function(child){
        var size = child.getContentSize();
        var width = size.width * child.getScaleX();
        var node = new cc.DrawNode();
        // var line1 = new cc.DrawNode();
        // var line2 = new cc.DrawNode();
        var color,lineColor;
        if(child.getTag()%10 === 0){
            color = cc.color(120,120,120,255);
            lineColor = cc.color(255,255,255,255);
        } else {
            color = cc.color(255,255,255,255);
            lineColor = cc.color(120,120,120,255);
            lineColor = cc.color(210,206,205,255);

        }
        node.drawRect(cc.p(child.x - width/2+8, child.y + size.height/2), cc.p(child.x + width/2-8, child.y + 93),  color, 0, color);
        node.drawRect(cc.p(child.x - width/2+8, child.y + size.height/2-2 + 50), cc.p(child.x + width/2-8, child.y + size.height/2 + 50), lineColor,0,lineColor);
        node.drawRect(cc.p(child.x - width/2+8, child.y + size.height/2-20 + 50), cc.p(child.x + width/2-8, child.y + size.height/2 - 18 + 50), lineColor,0,lineColor);
        return node;
    },
    createDecorate: function(clipper, child){
        var num = Utils.getRandomNum(3);
        var pos1 = [74,156,238,320,395,475];
        for(var i = 0;i < num; i += 1){
            var value = Utils.getRandomNum(Global.decorates.length) - 1;
            var data = Global.decorates[value];
            var sp = new cc.Sprite("#item/" + data.name + ".png");
            var width = Utils.getRandomNum(pos1.length) - 1;
            sp.setVisible(gameUserData.eye === 1);
            child.decorates = [];
            child.decorates.push(sp);
            this.addChild(sp, data.isShow* -1 - 1);
            var x = 0;
            if(child.getScaleX() < 0.9){
                x = child.x;
                sp.setPosition(cc.p(x , child.y + 7 + sp.getContentSize().height/2));
                i = num;
            } else {
                x = child.x + pos1[width]- child.getContentSize().width * child.getScaleX()/2;
                sp.setPosition(cc.p(x, child.y + 7 + sp.getContentSize().height/2));
                pos1.splice(width,1);
            }

            this.decorates.push(sp);
            if(data.isShow){
                var node = new cc.Sprite("#item/" + data.name + ".png");
                node.setPosition(cc.p(x, child.y + 7 + sp.getContentSize().height/2));
                clipper.stencil.addChild(node);
            }
        }
    },
    // 给地板设置碰撞矩形
    hitRectForFloor: function(){
        var floors = this.getFloorsNode().getChildren();
        for(var i = 0;i < floors.length; i += 1){
            floors[i]["gameRect"] = [];
            var size = floors[i].getContentSize();
            var width = size.width * floors[i].getScaleX();
            floors[i]["gameRect"] = cc.rect(0,0,width, size.height);
        }
    },
    // 给墙设置碰撞矩形
    hitRectForWall: function(){
        var walls = this.getWallsNode().getChildren();
        for(var i = 0; i < walls.length; i += 1){
            var size = walls[i].getContentSize();
            walls[i]["gameRect"] = cc.rect(0,0,size.width, size.height - 20);
        }
    },
    // 警告动画
    addWarningAction: function(){
        for(var i = 1; i <= 2; i += 1){
            var warningNode = this.getWallsNode();
            var warning = warningNode["wall_" + (2 + i)];
            warning["warningAction"] = new cc.Sprite("#bg/warning1.png");
            warning["warningAction"].setPosition(cc.p(30,97));
            warning.addChild(warning["warningAction"]);
            warning["warningAction"].setVisible(false);
            warning["warningAction"].runAction(AnimationMgr.getWarningAction().repeatForever());
        }
    }
});
