// 角色状态
var ROLE_STATE = {
    RUN: 0,    // 奔跑
    UP: 1,     // 向上
    DOWN:2,    // 向下
    LEFT:3,    // 向左
    RIGHT:4,   // 向右
    DIE: 5　　 // 死亡
};

// 默认是美国队长
var Role = cc.Node.extend({
    id: 0,  // 角色id
    state: 0, // 角色状态
    dir: ROLE_STATE.LEFT,   // 角色方向 左或者右
    role: null, // 角色
    data: null, // 角色数据
    boxRect: null, // 图片尺寸
    jumpTime: 0,  // 跳跃时间
    startPos: null, // 起始位置
    upTime: 0,  // 向上到最高点需要的时间
    upDistance: 0, // 上升距离
    parachute: null, // 降落伞
    isMaxHeight: false,  // 是否在最大高度
    isStop: false,
    roleSize: null, // 角色尺寸
    ctor: function(id){
        this._super();
        this.id = id || 0;

        this.jumpTime = 0;
        this.upTime = 0;
        this.startPos = null;
        this.data = Utils.getDataByID(GameRolesConfig, this.id);
        this.upDistance = 0;
        this.dir = ROLE_STATE.LEFT;
        this.state = ROLE_STATE.RUN;
        this.roleSize = null;
    },
    onEnter: function(){
        this._super();
        this.createRoleSP();
        this.roleDoAction();
        this.startPos = this.getPosition();
        this.getRoleBoundingBox();
        this.createParachute();
        this.schedule(function(){
            if(!Global.pause && this.state === ROLE_STATE.RUN ){
                SoundMgr.playEffect(soundRes.sound_run);
            }
        }.bind(this),0.3);
        // this.initData();
    },
    // 创建降落伞
    createParachute: function(){
        if(!this.parachute){
            this.parachute = new cc.Sprite("#item/parachute.png");
            // this.parachute.y = 105;
            this.parachute.attr({x:this.data.pos1.x, y:this.data.pos1.y});
            this.parachute.setVisible(false);
            this.addChild(this.parachute, -1);
        }
    },
    // 初始化数据
    initData: function(){
        this.startPos = this.getPosition();
        // cc.log(" role y = " + this.startPos.y);
        if(this.getRoleState() === ROLE_STATE.RUN){
            this.upDistance = this.data.h;
        } else {
            this.upDistance = 100;
        }
        this.upTime = 0;
        this.isMaxHeight = false;
        this.upTime = Utils.getMaxHeightTime(this.data.upG, this.upDistance);

    },
    getDownTime: function(){
        var temp = Utils.getMaxHeightTime(this.data.downG, 30);
        return;
    },
    // 创建角色精灵
    createRoleSP: function(){
        this.role = new cc.Sprite("#"+this.data.upImg);
        this.size = this.role.getContentSize();
        this.boxRect = cc.rect(0,0,this.size.width/2,this.size.height/2);
        this.role.setPosition(cc.p(this.size.width/2,this.size.height/2));
        // this.role.addChild(Utils.drawNode(this.boxRect));
        this.addChild(this.role);
    },
    // 播放动画
    roleDoAction: function(){
        switch(this.state){
            case ROLE_STATE.RUN:
                this.roleRunAction();
                break;
            case ROLE_STATE.UP:
                SoundMgr.playEffect(soundRes.sound_jump);
                this.roleUpAction();
                break;
            case ROLE_STATE.DOWN:
                this.roleDownAction();
                break;
            case  ROLE_STATE.DIE:  // 死亡降落伞
                SoundMgr.playEffect(soundRes.sound_die);
                break;
            default:
                cc.log("out of role state");
                break;
        }
        this.setRoleDir(this.dir);
    },
    // 设置状态
    setRoleState: function(state){
        if(this.state === state){ return;}
        if(state === ROLE_STATE.DIE){
            this.parachute.setVisible(true);
            if(this.dir === ROLE_STATE.LEFT){
                this.parachute.attr({x:this.data.pos1.x,y:this.data.pos1.y});
            }else{
                this.parachute.attr({x:this.data.pos2.x,y:this.data.pos2.y});
            }
            this.parachute.setFlippedX(this.dir !== ROLE_STATE.LEFT);
        } else {
            this.parachute.x = 0;
            this.parachute.setVisible(false);
        }
        this.state = state;
        this.jumpTime = 0;
        if(GameLogic.commands.length < 1){
            this.roleDoAction();
        }
    },
    // 获取状态
    getRoleState: function(){
        return this.state;
    },
    // 奔跑动画
    roleRunAction: function(){
        var action = Utils.getRoleAction(this.id);
        this.role.stopAllActions();
        this.role.runAction(action.repeatForever());
        this.state = ROLE_STATE.RUN;
    },
    // 上升动画
    roleUpAction: function(){
        this.role.stopAllActions();
        Utils.changeTexture(this.role, this.data.upImg);
        this.state = ROLE_STATE.UP;
    },
    // 下落动画
    roleDownAction: function(){
        this.role.stopAllActions();
        Utils.changeTexture(this.role, this.data.downImg);
        this.state = ROLE_STATE.DOWN;
    },
    // 获取角色的大小
    getRoleBoundingBox: function(){
        // var pos;
        // if(this.getParent()){
        //     pos = this.getParent().convertToWorldSpace(this.getPosition());
        // }
        // this.boxRect.x = this.x - this.boxRect.width/2;
        // this.boxRect.y = this.y - this.boxRect.height;
        this.boxRect.x = this.x + this.size.width/4;
        this.boxRect.y = this.y;
        // var rect = cc.rect(this.boxRect.x, this.boxRect.y,this.boxRect.width,this.boxRect.height/2);
        return this.boxRect;
    },
    // 设置角色的方向
    setRoleDir: function(dir){
        this.dir = dir;
        this.role.setFlippedX(dir !== ROLE_STATE.LEFT);
    },
    // 获取角色方向
    getRoleDir: function(){
        return this.dir;
    },
    // 获取向上的时间
    getUPTime: function(){
        return this.upTime;
    },
    // 水平移动
    horizontalMove: function(dt){
        var value = 1;
        if(this.state === ROLE_STATE.DIE){
            value = 0.6;
            // this.data.downG /= 2;
        }
        var factor = this.dir === ROLE_STATE.RIGHT? 1: -1;
        this.x += this.data.speedX * dt * factor * value;
    },
    // 角色动作
    update: function(dt){
        if(this.isStop){ return;}
        this.horizontalMove(dt);
        if(this.state == ROLE_STATE.UP){
            this.roleUp(dt);
        } else if(this.state === ROLE_STATE.DOWN || this.state === ROLE_STATE.DIE){
            this.roleDown(dt);
        }
    },
    // 上升
    roleUp: function(dt){
        this.jumpTime += dt;
        var per = this.jumpTime/this.upTime;
        var value = this.data.upG * this.jumpTime;
        var h = this.upTime * value - 0.5 * this.jumpTime * value;
        this.y = this.startPos.y + h;
        if(per >= 1){
            this.isMaxHeight = true;
            this.setRoleState(ROLE_STATE.DOWN);
            this.jumpTime = 0;
            this.y = this.startPos.y + this.upDistance;
        }
    },
    cleanData: function(){
        this.startPos.y = this.y;
        this.setRoleState(ROLE_STATE.UP);
        this.jumpTime = 0;
        this.upTime = Utils.getMaxHeightTime(this.data.upG, 75);
        this.isMaxHeight = false;
    },
    // 下降
    roleDown: function(dt){
        this.isMaxHeight = false;
        this.jumpTime += dt;
        var distance = 0;
        if(this.state === ROLE_STATE.DIE){
            distance =  this.data.speedX * dt * 0.7;
            this.y -= distance;
        } else {
            distance = 0.5*this.data.downG * this.jumpTime * this.jumpTime;
            this.y = this.startPos.y + this.upDistance - distance;
        }
    },
    // 下落时间
    getDownTime: function(){
        return Utils.getMaxHeightTime(this.data.downG, 25);
    },
    replayRole: function(){
        this.role.removeFromParent(true);
        this.jumpTime = 0;
        this.parachute = null;
        this.id = gameUserData.curRole;
        this.data = Utils.getDataByID(GameRolesConfig, this.id);
        this.upDistance = 0;
        this.dir = ROLE_STATE.LEFT;

        this.createRoleSP();
        this.createParachute();
        this.roleDoAction();
    }
});