
var CHOOSE_TYPE = {
    CHOOSE_ROLE: 0,
    CHOOSE_SCENE: 1
};

// 选中场景和角色
var ChooseLayer = cc.Layer.extend({
    mainUI: null,
    language: null,
    chooseRoleID: 1002,
    chooseSceneID: 1002,
    beforeIndex: 0,
    ctor: function(){
        this._super();
        this.language = Global.language || "en";
        this.chooseSceneID = gameUserData.curScene;
        this.chooseRoleID = gameUserData.curRole;
    },
    onEnter: function(){
        this._super();
        SoundMgr.playMusic(soundRes.sound_choose_role, true);
        this.initUI();
        this.initListener();
        this.initText();
        this.scheduleUpdate();
        Event.register(EVENT_ID.CHOOSEREFRESHCOINS, function(){
            this.initText();
        }, this);
    },
    onExit: function(){
        this._super();
        Event.remove(EVENT_ID.CHOOSEREFRESHCOINS, this);
    },
    initUI: function(){
        this.mainUI = loadCSS(res.res_game_choose_layer);
        this.addChild(this.mainUI);

        this.createRoleListView(CHOOSE_TYPE.CHOOSE_ROLE);
        this.createRoleListView(CHOOSE_TYPE.CHOOSE_SCENE);
    },
    initListener: function(){
        engine.gameEvent.createMouseUp(this.mainUI.btnHome, this.onHome.bind(this), buttonScalingType);
    },
    initText: function(){
        this.mainUI.goldText.setString(gameUserData.gold);
    },
    // 根据类型选择数据
    getDataByType: function(type){
        var data;
        if(type === CHOOSE_TYPE.CHOOSE_ROLE){
            data = GameRolesConfig;
        } else {
            data = GameSceneConfig;
        }
        return data;
    },
    onChange: function(sender, type){
        cc.log("touch type : type " + type);
        sender.getParent().setItemScale(sender);
        if(type === ccui.ScrollView.EVENT_CONTAINER_MOVED){
            sender.getParent().setItemScale(sender);
        }
    },
    // 选中的角色和场景
    chooseRoleAndScene: function(list, index){
        if(list.gameChooseType === CHOOSE_TYPE.CHOOSE_ROLE){
            if(this.chooseRoleID !== GameRolesConfig[index].id){
                this.chooseRoleID = GameRolesConfig[index].id;
                SoundMgr.playEffect(soundRes.sound_choose);
            }
        } else {
            if(this.chooseSceneID !== GameSceneConfig[index].id){
                SoundMgr.playEffect(soundRes.sound_choose);
                this.chooseSceneID = GameSceneConfig[index].id;
            }
        }
    },
    // 获取中间的index
    getSelectedIndex: function(list, type){
        type = type || 0;
        var index = 0;
        if(type === 1){
            index = 0;
            if(list.gameChooseType === CHOOSE_TYPE.CHOOSE_ROLE){
                list.jumpToItem(gameUserData.curRole - 1000, cc.p(0.5,0.5), cc.p(0.5,0.5));
            } else {
                list.jumpToItem(gameUserData.curScene - 2000, cc.p(0.5,0.5), cc.p(0.5,0.5));
            }

            this.chooseRoleID = gameUserData.curRole;
            this.chooseSceneID = gameUserData.curScene;
        } else {

            var pTargetItem = list.getCenterItemInCurrentView();
            index = list.getIndex(pTargetItem);
            this.chooseRoleAndScene(list, index);
        }
        return index;
    },
    // 设置item的缩放效果
    setItemScale: function(list, type){
        // var pTargetItem = list.getClosestItemToPosition(cc.p(590, 430), cc.p(0.5, 0.5));
        var index = this.getSelectedIndex(list, type);
        var items = list.getItems();  // 获取所有item

        for(var i = 0; i < items.length; i += 1){
            if(i === index){
                // cc.log(" index = " + i);
                var scale = Math.abs(items[i].x + items[i].getParent().x - 295) * 0.0013;
                items[i].setScale(1 - scale);
                items[i].roleName.setVisible(true);
            } else {
                var scale = Math.abs(items[i].x + items[i].getParent().x - 295) * 0.0013;
                // items[i].setScale(1 - (Math.abs(index - i))*0.2);
                items[i].setScale(1 - scale);
                items[i].roleName.setVisible(false);
            }
        }
    },
    update: function(dt){
        var node1 = this.getChildByTag(CHOOSE_TYPE.CHOOSE_ROLE);
        var node2 = this.getChildByTag(CHOOSE_TYPE.CHOOSE_SCENE);
        this.setItemScale(node1);
        this.setItemScale(node2);
    },
    // 角色滑动框
    createRoleListView: function(type){
        var listView = new ccui.ListView();
        listView.setScrollBarEnabled(false);
        listView.setDirection(ccui.ScrollView.DIR_HORIZONTAL);
        listView.setContentSize(cc.size(590, 430));
        listView.setBounceEnabled(true);
        listView.setMagneticType(ccui.ListView.MAGNETIC_CENTER);
        listView.setGravity(ccui.ListView.GRAVITY_LEFT);
        // listView.addEventListener(this.onChange, this);
        // listView.addTouchEventListener(this.onChange, this);
        listView.gameChooseType = type;
        if(type === CHOOSE_TYPE.CHOOSE_ROLE){
            listView.attr({x:78, y:723});
        } else {
            listView.attr({x:78, y:273});
        }

        var data = this.getDataByType(type);
        for(var i = 0; i < data.length; ++i){
            var item = this.createRoleItem(data[i]);
            listView.addChild(item);
        }
        listView.setTag(type);
        this.addChild(listView);
        this.setItemScale(listView, 1);
    },
    // 创建角色item
    createRoleItem: function(data){
        // var data = GameRolesConfig[index];
        // data.id = 1001;
        var item = new ccui.Layout();
        item.setAnchorPoint(cc.p(0.5, 0.5));
        item.setContentSize(198, 405);

        item.frame = new cc.Sprite("#choose_role_and_scene/choose_frame.png");
        item.frame.attr({x: 94, y: 213});
        item.addChild(item.frame);

        item.role = new cc.Sprite();
        item.role.attr({x: 94, y: 180});
        if(data.id > 2000){
            item.role.setScale(0.8);
            item.role.y = 205;
            Utils.changeTexture(item.role, data.icon);
        } else if(data.id === 2000 || data.id === 1000) {
            Utils.changeTexture(item.role, data.icon);
        } else {
            item.role.setScale(2);
            item.role.runAction(Utils.getRoleAction(data.id).repeatForever());
        }
        item.addChild(item.role);

        if(( data.id > 1000 && data.id < 2000 && !GameLogic.isInArray(gameUserData.roles, data.id))
            || (data.id > 2000 && !GameLogic.isInArray(gameUserData.scenes, data.id))){
            item.lockFrame = new cc.Sprite("#choose_role_and_scene/lock_frame.png");
            item.lockFrame.attr({x: 94, y: 213});
            item.addChild(item.lockFrame);

            item.button = new ccui.Button();
            item.button.loadTextureNormal("choose_role_and_scene/price_frame.png", ccui.Widget.PLIST_TEXTURE);
            item.button.setTag(data.id);
            item.button.setTouchEnabled(true);
            // item.button.setBright(false);
            item.button.addClickEventListener(this.onBuy.bind(this));
            item.button.attr({x: 94, y: 47});
            item.button.setTitleText(data.price);
            item.button.setTitleFontSize(40);
            item.button.setTitleColor(cc.color(255,255,255,255));
            item.addChild(item.button);

            if(data.id - gameUserData.scenes.length - 2000 > 1){
                item.button.setVisible(false);
            }
        }

        item.roleName = new cc.LabelTTF(data[this.language],"Arial", 36, cc.size(370, 100),  cc.TEXT_ALIGNMENT_CENTER, cc.VERTICAL_TEXT_ALIGNMENT_CENTER);
        item.roleName.attr({x:94, y: 378});
        item.addChild(item.roleName);

        return item;
    },
    // 刷新购买的item
    refreshBuyItem: function(item){
        item.button.setVisible(false);
        item.lockFrame.setVisible(false);
        this.initText();
        if(item.button.getTag() > 2000){
            this.openNextScene(item.getParent().getParent(), item.button.getTag());
        }

    },
    // 购买了场景之后开启下一个场景
    openNextScene: function(list, id){
        var item = list.getItem(id - 1999);
        item.button.setVisible(true);
    },
    onBuy: function(sender, type){
        cc.log("on buy !!!");
        var id = sender.getTag();
        var data;
        if(id >= 2000){
            data = Utils.getDataByID(GameSceneConfig, id);
        } else {
            data = Utils.getDataByID(GameRolesConfig, id);
        }

        if(!data){ return ;}
        if(gameUserData.gold >= data.price){   // 购买角色或者场景
            if(id >= 2000 ){  // 购买场景
                gameUserData.scenes.push(id);
            } else {       // 购买角色
                gameUserData.roles.push(id);
            }
            gameUserData.gold -= data.price;
            Utils.removeDataFromArray(gameUserData.lotterys, id);
            saveUserData();
            this.refreshBuyItem(sender.getParent());
        } else {
            cc.log("no rich enough to buy this item !!!");
            this.showLakeCoins();

        }
    },
    showLakeCoins:function(){
        var layer = new LakeConinsLayer();
        createMaskLayer(layer);
        this.getParent().addChild(layer,2001);
    },
    onHome: function(){
        SoundMgr.playEffect(soundRes.sound_btn);
        if(GameLogic.isInArray(gameUserData.roles, this.chooseRoleID)){
            gameUserData.curRole = this.chooseRoleID;
        }
        if(GameLogic.isInArray(gameUserData.scenes, this.chooseSceneID)){
            gameUserData.curScene = this.chooseSceneID;
        }
        if(this.chooseRoleID === 1000){
            gameUserData.curRole = 1000;
        }
        if(this.chooseSceneID === 2000){
            gameUserData.curScene = 2000;
        }
        saveUserData();
        GameLogic.reset();
        Global.historyLevel = 0;
        Global.replayNum = 0;

        this.removeFromParent(true);
        Global.pause = false;
        Event.emit(EVENT_ID.REPLAY);
        // transitions(HomeScene);
    }
});