var GAME_TAG = {
    LOSE: 12,
    CONPLAY: 11,
    GAME_MENU: 10,
    LEFT_UP: 9,
    PAUSE: 8,
    ROLE: 7,
    OBSTACLE: 6,
    SNOW:5,
    BG:4, // 背景层
    SKY:3 // 天空层
};

// 游戏层
var GameFightLayer = cc.Layer.extend({
    isFirstTouch: true,
    obstacleNode: null, // 障碍物
    role: null,   // 角色
    isCanTouch: true, // 表示是否点击了
    isCheckDie: false, // 检测是否死亡
    maps: [],      // 地图
    floors:[],     // 地板
    walls:[],      // 墙
    golds: [],    // 金币
    mapStartPos: null,
    bgStartPos: null,
    num: 0,       // 地图的数量
    isMoveMap: false, // 是否移动地图
    mapMoveTime: 0,  // 地图移动时间
    maxHeight: 0, // 跳的最大高度
    mapStartPosX: null, // 水平的启动
    bgStartPosX: null,  // 背景的启动
    roleDistanceCenterX: 0, // 角色与中心点的x距离
    bgs: [],            // 背景
    bgNode: null,       // 背景层
    bgFact: 0.3,
    outScreenBG: null,  // 屏幕外面的背景
    isLetter: false,     // 是否领取过信封
    isStartMoveH: false, // 是否开启水平位移
    distanceH: 0,        // 水平位移距离
    isFixRoleH: false,   // 矫正水平位置
    isBgOut: false,      // 背景达到最高点后隐藏
    backHomeNum: 2, // 回家次数2次就重玩
    fastMoveTime: 0,  // 水平快速移动时间
    emitter: null,   //  特效层
    startLevel: 1,   //  起始层
    hitWalls: [],    // 需要碰撞检测的墙壁
    hitFloors: [],   // 需要碰撞检测的地板
    curSceneID: 0,   // 当前场景的id
    stars:[],        // 夜空中的星
    sky: null,       // 天空
    clouds:[],        // 云朵
    isShowCloud: false, // 是否已经显示过云了
    ctor: function(){
        this._super();
        this.initData();
        this.registerEvent();
        this.backHomeNum = 2;
    },
    initData: function(){
        this.isLetter = false;
        this.isMoveMap = false; // 是否移动地图
        this.mapMoveTime =  0;  // 地图移动时间
        this.mapStartPosX = null; // 水平的启动
        this.bgStartPosX = null;  // 背景的启动
        this.outScreenBG = null;
        this.isStartMoveH = false; // 是否开启水平位移
        this.distanceH =  0;        // 水平位移距离
        this.isCheckDie =  false;
        this.isFirstTouch = true;
        this.num = 0;
        this.horizontalTime = 0;
        this.isBgOut = false;
        this.fastMoveTime = 0;
        GameLogic.commands = [];
        this.hitWalls = [];
        this.hitFloors = [];
        this.curSceneID = 0;
        this.randomCloudLevel();
    },
    onEnter: function(){
        this._super();
        this.initUI();
        // this.role.isStop = true;
        if(!GameLogic.curFloor){
            GameLogic.curFloor = this.maps[0].getFloorsNode().getChildByName("floor_" + 1);
        }
        this.refreshFloorsArray();
        this.refreshWallsArray();
        this.disableOutScreen();

         //this.addChild(Utils.drawNode(cc.rect(640, 266, 35,5)),1001);
         //this.addChild(Utils.drawNode(cc.rect(360, 266, 2,35)), 1001);

        this.createSky();
        this.createEmitter();
        this.scheduleOnce(function(){
            this.scheduleUpdate();
        }.bind(this), 0.1);
    },
    // 天空
    createSky: function(){
        if(this.sky !== null){
            this.sky.removeFromParent();
        }
        this.sky = new cc.Node();
        this.sky.y = 0;
        var data = SkyColorConfig[this.curSceneID - 2001];
        for(var i = 0; i < data.length; i += 3){
            var layer1 = new cc.LayerGradient(cc.color(data[i],data[i + 1], data[i + 2], 255), cc.color(data[i+3], data[i+4], data[i+5], 255),cc.p(0,1));
            layer1.setContentSize(cc.size(720, 640));
            layer1.setPosition(cc.p(0, 640 * (i/3) - 5 *(i/3)));
            this.sky.addChild(layer1, GAME_TAG.SKY, 0);
        }
        // this.sky.setVisible(false);
        this.addChild(this.sky, GAME_TAG.SKY);
    },
    // 粒子效果
    createEmitter: function(){
        if(Global.isFisrtEnter){
            Global.isFisrtEnter = false;
            return;
        }
        var value = Utils.getRandomNum(10);
        if(this.emitter !== null){
            this.emitter.removeFromParent();
            this.emitter = null;
        }
        if(value <= 2){
            this.emitter = AnimationMgr.rainDrop();
            // this.emitter.setVisible(false);
            this.addChild(this.emitter, GAME_TAG.SNOW);
        } else if(value >= 8){
            this.emitter = AnimationMgr.snowDance();
            // this.emitter.setVisible(false);
            this.addChild(this.emitter, GAME_TAG.SNOW);
        }
    },
    // 注册消息
    registerEvent: function(){
        Event.register(EVENT_ID.CHECK_RESULT, function(){
            this.addResultUITwo();
        }, this);

        Event.register(EVENT_ID.REPLAY, function(){
            this.replay();
        }, this);
        Event.register(EVENT_ID.REPLAYCOINS, function(){
            this.replayCoins();
        }, this);
    },
    onExit: function(){
        this._super();
        Event.remove(EVENT_ID.CHECK_RESULT, this);
        Event.remove(EVENT_ID.REPLAY, this);
        Event.remove(EVENT_ID.REPLAYCOINS, this);
    },
    initUI: function(){
        // this.addChild(new MapObstacle(), GAME_TAG.OBSTACLE);
        this.createObstacleFatherNode();
        this.createObstacleNode();
        this.createRole();
        this.createBg();

        this.addChild(new GameMenuLayer(), GAME_TAG.GAME_MENU);
        this.addChild(new LeftAndUPLayer(), GAME_TAG.LEFT_UP);
        engine.gameEvent.createMouseDown(this, this.onTouch.bind(this));
    },
    // 地图数据
    refreshMapData: function(){
        if(Global.curLevel >= Global.hards[1]){
            Global.maps[0] = 1031;
            Global.maps[1] = 1032;
        } else if(Global.curLevel >= Global.hards[0]){
            Global.maps[0] = 1021;
            Global.maps[1] = 1022
        } else {
            Global.maps[0] = 1011;
            Global.maps[1] = 1012;
        }

        //if(Global.curLevel >= Global.hards[0] && Global.curLevel < Global.hards[1]){
        //    Global.maps[0] = 1021;
        //    Global.maps[1] = 1022;
        //} else if(Global.curLevel >= 200){
        //    Global.maps[0] = 1031;
        //    Global.maps[1] = 1032
        //} else {
        //    Global.maps[0] = 1011;
        //    Global.maps[1] = 1012;
        //}
    },
    // 创建障碍物
    createObstacleFatherNode: function(){
        this.obstacleNode = new cc.Node();
        this.obstacleNode.attr({x:88, y:300});
        this.addChild(this.obstacleNode, GAME_TAG.OBSTACLE);
        GameLogic.beforeType = 0;
        GameLogic.loadMap(this.obstacleNode);
    },
    // 创建关卡
    createObstacleNode: function(){
        this.num = Math.floor(Global.curLevel/10);
        this.refreshMapData();

        for(var i = 0 ;i < Global.maps.length; i += 1){
            var id = GameLogic.getMapId();
            if(Global.maps[i] > 0){
                id = Global.maps[i];
            }
            var map = GameLogic.getMapNodeBy(id);

            map.setVisible(true);
            map.num = this.num;
            map.refreshFloor();
            this.num += 1;
            // var value = GameLogic.getGoldNum();
            var value = Utils.getRandomNum(4) + 4;
            map.randGoldPosition(value);
            map.setName("level" + (i + 1));
            if(i > 0){
                var lastFloor = this.maps[i - 1].getLastFloor();
                map.setPosition(cc.p(this.maps[i-1].x + lastFloor.x - 272 , lastFloor.y + this.maps[i - 1].y + 93 ));
             }else {
                map.setPosition(cc.p(0 , 0 ));
            }

            this.maps[i] = map;
            this.maps[i]["mapIndex"] = i;

        }
    },
    // 创建地图的地板
    createMapFloor: function(id){
        var data = Utils.getDataByID(GameSceneConfig, id);
        var sprite = new cc.Sprite("#" + data.mapFloor);
        sprite.setPosition(cc.p(sprite.getContentSize().width/2 - this.obstacleNode.x, sprite.getContentSize().height/2 - this.obstacleNode.y));
        this.obstacleNode.addChild(sprite, -1);
    },
    // 创建背景层
    createBg: function(){
        this.bgNode = new cc.Node();
        this.bgs = [];
        this.addChild(this.bgNode, GAME_TAG.BG, GAME_TAG.BG);
        var id = gameUserData.curScene;
        if(id === 2000){
            id = Utils.randomFromArray(gameUserData.scenes);
        }
        for(var i = 0; i <= 1; i += 1){
            this.bgs[i] = new BackgroundLayer(id);
            // this.bgs[i].setVisible(false);
            this.bgs[i].attr({x: -this.bgs[i].getBgSize().width * i,y:0});
            // this.bgs[i].setVisible(false);
            this.bgNode.addChild(this.bgs[i]);
        }
        this.createMapFloor(id);
        this.curSceneID = id;
    },
    // 创建角色
    createRole: function(){
        var id = gameUserData.curRole;
        if(gameUserData.curRole === 1000){
            id = Utils.randomFromArray(gameUserData.roles);
        }
        this.role = new Role(id);
        this.role.attr({x: 272, y:10});
        this.obstacleNode.addChild(this.role, GAME_TAG.ROLE);

        var positionWorld=this.obstacleNode.convertToWorldSpace(cc.p(272,10));
        cc.log(positionWorld.x+"  KKKKKKKKKKKKK  "+positionWorld.y);
        // this.addChild(Utils.drawNode(this.role.getRoleBoundingBox()));
        // this.addChild(this.role, GAME_TAG.ROLE);
    },
    // 创建暂停界面
    createPauseLayer: function(){
        var layer = new PauseLayer();
        createMaskLayer(layer);
        this.addChild(layer, GAME_TAG.PAUSE);
    },
    // 发送消息
    emitEvent: function(){
        if(this.isFirstTouch){
            this.isFirstTouch = false;
            // this.role.isStop = false;

            Event.emit(EVENT_ID.START_GAME);
            Event.emit(EVENT_ID.PAUSE_IN_OUT);
        }
    },
    temp:0,
    update: function(dt){
        this.temp++;
        this.temp = this.temp > 1? 0:this.temp;
        if(Global.pause || (this.temp === 1 && this.checkResult())){
            return;
        }

        if(GameLogic.controlRole(this.role) && this.isMoveMapV() && this.role.getRoleState() !== ROLE_STATE.DIE){
            GameLogic.maxFloor = GameLogic.curFloor;
            this.maxHeight = GameLogic.curFloor.y;
            this.isMoveMap = true;
            this.isCheckDie = false;
            this.mapStartPos = this.obstacleNode.getPosition();
            this.bgStartPos = this.bgNode.getPosition();
            this.freshMapData();
            this.refreshFloorsArray();
            this.refreshWallsArray();
            this.refreshGold();

            this.createCloud();
            return;
        }
        this.role.update(dt);
        if(this.role.getRoleState() === ROLE_STATE.DIE){
            return;
        }

        this.ctrlRoleState();
        this.ctrlMapMove(dt);
    },
    // 角色状态
    ctrlRoleState: function(){
        GameLogic.checkGold(this.role, this.golds);
        if(GameLogic.checkRoleAndWalls(this.role,  this.getWallsArray()) && this.role.getRoleState() !== ROLE_STATE.RUN){     // 检测是否与墙壁碰撞
            this.isCheckDie = false;
        }


        if(this.isCheckDie && this.temp === 0&& GameLogic.nextFloor !== null && this.roleIsUnderFloor(this.role, GameLogic.nextFloor)){
            this.isCheckDie = false;
        }

        if(!this.isCheckDie && this.role.getRoleState() === ROLE_STATE.DOWN){   // 检测下降的时候，是否死亡
            this.isCheckDie = true;
            this.refreshFloorsArray();
            this.refreshWallsArray();
            this.disableOutScreen();
            if(GameLogic.checkRoleIsDie(this.role, this.getFloorsArray()) && !GameLogic.checkRoleWillHitWall(this.role, this.getWallsArray())){   // 检测是否死亡
                this.role.startPos = this.role.getPosition();
                this.role.setRoleState(ROLE_STATE.DIE);
                return;
            }
            return;
        }

        if(this.role.getRoleState() === ROLE_STATE.DOWN &&GameLogic.checkFloorUnderRole(this.role, this.getFloorsArray()) ){ // 下落的时候检测 与地面碰撞
            // GameLogic.curFloor = GameLogic.nextFloor;
            this.role.startPos = this.role.getPosition();
            this.role.setRoleState(ROLE_STATE.RUN);
        } else if(this.temp === 0 && this.role.getRoleState() === ROLE_STATE.RUN && !GameLogic.checkRoleHit(this.role, GameLogic.curFloor) /*&& !GameLogic.checkFloorUnderRole(this.role, this.getFloorsArray() )*/){// 奔跑的时候检测
            this.role.upDistance = 0;
            this.role.setRoleState(ROLE_STATE.DOWN);
            this.isCheckDie = false;
        }
    },
    // 地图移动
    ctrlMapMove: function(dt){
        // 地图的移动
        if(this.isStartMoveH){
            this.moveMapHorizontal(dt);
        }
        if(this.curMap().type === MAP_TYPE.H && !this.isStartMoveH && !this.curMap().isMoveEnd){ // 水平地图
            this.moveMapH(dt)
        }else if(this.isMoveMap){  // 是否移动地图
            this.moveMapV(dt);  // 垂直地图
        }
    },
    // 金币
    refreshGold: function(){
        this.golds = this.curMap().golds;
        for(var i = 0; i < this.golds.length; i += 1){
            if(this.golds[i]){
                var pos = cc.pAdd(this.curMap().getPosition(), this.golds[i].getPosition());
                this.golds[i]["gameRect"].x = pos.x - this.golds[i]["gameRect"].width/2;
                this.golds[i]["gameRect"].y = pos.y- this.golds[i]["gameRect"].height/2;
            }
        }

    },
    // 获取墙
    getWallsArray: function(){
        // return this.hitWalls;
        return this.walls;
    },
    refreshWallsArray: function(){
        var sum = 0;
        this.walls = [];
        for(var i = 0; i < this.maps.length; i += 1){
            var map = this.maps[i];
            var children = map.getWallsNode().getChildren();
            for(var j = 0; j < children.length; j += 1){
                var pos = cc.pAdd(children[j].getParent().getParent().getParent(), children[j].getPosition());
                children[j]["gameRect"].x = pos.x - children[j]["gameRect"].width/2;
                children[j]["gameRect"].y = pos.y - children[j]["gameRect"].height/2;
                this.walls[sum++] = children[j];
            }
        }
    },
    // 获取 地
    getFloorsArray: function(){
        // return this.hitFloors;
        return this.floors;
    },
    refreshFloorsArray: function(){
        var index = 0;
        for(var i = this.maps.length - 1; i >= 0; i -= 1){
            var node = this.maps[i].getFloorsNode();
            var children = node.getChildren();
            for(var j = 0; j < children.length; j += 1){

                var child = node.getChildByName("floor_" + (children.length - j));
                var pos = cc.pAdd(child.getPosition(), child.getParent().getParent().getParent().getPosition());
                child["gameRect"].x = pos.x -  child["gameRect"].width/2;
                child["gameRect"].y = pos.y - child["gameRect"].height/2;

                this.floors[index] = child;
                index += 1;
            }
        }
    },
    // 弹出sdk
    showSDKUI: function () {
        if(!Global.isPure && gameUserData.maxLevel >= gSdkData.unlockLevel - 1 && !gameUserData.isLockState){
            SDKControl.addSDKView(this);
            return true;
        }
        return false;
    },
    // 点击屏幕
    onTouch: function(touch, event){
        if(this.showSDKUI()){ return ;}
        if(ROLE_STATE.DIE === this.role.getRoleState() || !this.isCanTouch){
            return;
        }
        if(this.curMap().type === MAP_TYPE.H && ROLE_STATE.RUN !== this.role.getRoleState() ){
            return;
        }
        GameLogic.pushCommand();
        this.isCheckDie = false;
        this.emitEvent();
    },
    // 检测是否失败
    checkResult: function(){
        /*if(this.role.getRoleState() === ROLE_STATE.RUN){
            return;
        }*/
        var winSize = cc.director.getWinSize();
        var pos = this.obstacleNode.convertToWorldSpace(this.role.getPosition());

        if((pos.x > winSize.width  + 50
            || pos.x < -this.role.size.width - 50
        || pos.y < -this.role.size.height - 60)){

            if(this.backHomeNum > 0 && this.backToHome()){ return ;}
            if(Global.curLevel > Global.historyLevel && Global.curLevel >= 100){
                Global.historyLevel = Global.curLevel;
                Global.replayNum = 0;
            }
            this.addResultUI();
            Event.emit(EVENT_ID.REFRESH_GOLD);
            Global.pause = true;
            this.unscheduleUpdate();
            return true;
        }
        return false;
    },
    checkArch:function(){
        for(var i = 0; i < TaskExtraConfig.length; i++){
            if(gameUserData.archTasks[i]!=1){
                var id = getArchState(i);
                if(id==1){
                    googleUploadArch(i+1);
                    return;
                }
            }
        }
    },
    // 添加失败界面
    addResultUI: function(){
        showFacebookAd();
        googleUploadLead(Global.curLevel);
        this.checkArch();
        this.addContinuePlay();
        //this.addResultUITwo();
    },
    addResultUITwo:function(){
        if(gameUserData.gold >= Global.messagePrice && gameUserData.lotterys.length >= 1 && !this.isLetter){  // 金币足够就弹信封
            this.isLetter = true;
            this.addLetterUI();
            return;
        }

        this.addLoseUI();
        return;

        //if(Global.replayNum <= 2){
        //    this.addLoseUI();
        //    return;
        //}
        //
        //if(Global.historyLevel >= 100 && Global.replayNum > 2){
        //    this.addResurgenceUI();
        //}
    },
    addContinuePlay:function(){
        var layer = new ContiniuplayLayer();
        createMaskLayer(layer);
        this.addChild(layer,GAME_TAG.CONPLAY);

    },
    // 复活询问
    addResurgenceUI: function(){
        //var layer = new ResurgenceLayer();
        //createMaskLayer(layer);
        //this.addChild(layer,GAME_TAG.LOSE);
    },
    // 结束界面
    addLoseUI: function(){
        var layer = new ResultLayer(this.curSceneID);
        createMaskLayer(layer);
        this.addChild(layer,GAME_TAG.LOSE);
    },
    // 添加信封界面
    addLetterUI: function(){
        var layer = new LetterLayer();
        createMaskLayer(layer);
        this.addChild(layer,GAME_TAG.LOSE);
    },
    // 地图是否超出边界
    isMapOutScreen: function(){
        var length = this.maps[0].getFloorsNode().getChildren().length;
        var floor = this.maps[0].getFloorsNode().getChildByName("floor_" + length);
        var pos = cc.pAdd(cc.pAdd(this.obstacleNode.getPosition(), this.maps[0].getPosition()), floor.getPosition());
        if(pos.y < -50 || pos.x < -360 || pos.x > 1080){
            return true;
        }
        return false;
    },
    // 刷新地图数据
    freshMapData: function(){
        if(Global.curLevel - Global.historyLevel >= Global.starLevel && Global.curLevel%10 === 0){
            this.createStar();
        }

        if( Global.curLevel - this.startLevel >= Global.resurgenceLevel
        && Global.curLevel % 5 === 0 && this.curMap().mapIndex !== 0&& this.isMapOutScreen() ){
            this.maps[0].setVisible(false);
            this.maps[0].setPosition(cc.p(0,-3000));
            this.maps[0] = this.maps[1];
            this.maps[0]["mapIndex"] = 0;
            this.maps[1] = this.maps[2];
            this.maps[1]["mapIndex"] = 1;
            var num = this.maps[1].num + 1;

            var id = GameLogic.getMapId();
            var map = GameLogic.getMapNodeBy(id);

            cc.log("map ID = " + id + " num = " + num);

            this.refreshFloorsArray();
            this.refreshWallsArray();
            this.disableOutScreen();

            map.setVisible(true);
            map.num = num;
            map.refreshFloor();
            var value = GameLogic.getGoldNum((num  + 1)* 10);
            map.randGoldPosition(value);

            map.setName("level" + (num));
            var lastFloor = this.maps[1].getLastFloor();
            var x = this.maps[1].x + lastFloor.x - 272, y = this.maps[1].getLastFloor().y + this.maps[1].y + 93;
            var pos = cc.p(x,y);
            map.setPosition(pos);
            // this.obstacleNode.addChild(map);
            this.maps[2] = map;
            this.maps[2]["mapIndex"] = 2;

            // this.createCloud(map,pos);

        }
    },
    // 当前所在的地图
    curMap: function(){
        if(GameLogic.curFloor){
            var map = GameLogic.curFloor.getParent().getParent().getParent();
            return map
        }
        return null;
    },
    // 垂直地图移动
    moveMapV: function(dt){
        var map = this.curMap();
        if(!map){ return; }
        if(this.sky.y > -1270){
            this.sky.y -= dt * Global.skyMoveSpeed;
            // this.sky.y = -1280
        }

        this.mapMoveTime += dt;
        var per = this.mapMoveTime/(this.role.upTime);
        if(per >= 1){
            this.mapMoveTime = 0;
            this.isMoveMap = false;
            this.obstacleNode.y = this.mapStartPos.y - map.getMapHeight();

            this.bgNode.y = this.bgStartPos.y - map.getMapHeight() * this.bgFact;
            if(GameLogic.curFloor.getTag()%10 !== 0 && !this.isStartMoveH && map.type === MAP_TYPE.V){
                this.obstacleNode.x = this.mapStartPos.x + map.getMapWidth();

                this.bgNode.x = this.bgStartPos.x + map.getMapWidth() * this.bgFact;
                this.refreshBgPosition(map.getMapWidth() * this.bgFact);
            }
        } else{
            var value = per * map.getMapHeight();
            var  bgValue = value * this.bgFact;

            this.obstacleNode.y = this.mapStartPos.y - value;
            this.bgNode.y = this.bgStartPos.y - bgValue;

            if(GameLogic.curFloor.getTag()%10 !== 0 && !this.isStartMoveH && map.type === MAP_TYPE.V){ // 最后一个x轴不移动
                var xValue =  per * map.getMapWidth();
                this.obstacleNode.x = this.mapStartPos.x +  xValue;
                this.bgNode.x = this.bgStartPos.x + xValue * this.bgFact;
                this.refreshBgPosition( xValue * this.bgFact);
            }
        }
    },
    // 水平移动
    moveMapH: function(dt){
        var map = this.curMap();
        if(!map || map.getMapDir() === this.role.getRoleDir() || this.role.getRoleState() !== ROLE_STATE.RUN){ return; }
        this.isStartMoveH = true;
        this.mapStartPosX = this.obstacleNode.x;
        this.distanceH = map.moveDistance;
        map.isMoveEnd = true;
        this.bgStartPosX = this.bgNode.x;
        this.isFixRoleH = false;
    },
    horizontalTime: 0,
    // 地图水平移动
    moveMapHorizontal: function(dt){
        this.horizontalTime += dt;
        var fact = 1;
        if(this.distanceH <= 0){
            fact = -1
        }
        var speed = this.role.data.speedX;
        var num = this.isRoleInCenter();
        if(!this.isFixRoleH ){
            speed = this.role.data.speedX + 75 * num * fact * -1;
        }
        // var distance = this.horizontalTime*speed*fact + this.fastMoveTime*35;
        // this.obstacleNode.x = this.mapStartPosX + distance;
        var distance = dt*speed*fact;
        this.obstacleNode.x += distance;
        this.bgNode.x += distance*this.bgFact;
        this.refreshBgPosition(distance*this.bgFact);
        if(Math.abs(this.obstacleNode.x - this.mapStartPosX) >= Math.abs(this.distanceH)){
            this.obstacleNode.x = this.mapStartPosX + this.distanceH;

            this.bgNode.x = this.bgStartPosX + this.distanceH*this.bgFact;
            this.refreshBgPosition(this.distanceH*this.bgFact);
            this.isStartMoveH = false;
            this.horizontalTime = 0;
        }
    },
    // 水平移动的时候检测角色是否在中间
    isRoleInCenter: function(){
        var pos = this.obstacleNode.convertToWorldSpace(this.role.getPosition());
        if(pos.x > 390){
            // this.role.isStop = true;
            return 1;
        } else if (pos.x < 330){
            return -1;
        }
        this.isFixRoleH = true;

        return false;
    },
    // 是否移动地图
    isMoveMapV: function(){
        var rolePos = this.obstacleNode.convertToWorldSpace(this.role.getPosition());
        // cc.log("rolePos. y = " + rolePos.y );
        if(rolePos.y > 306){
            return true;
        } else {
            return false;
        }
    },
    // 屏幕外的背景
    findOutScreenBg: function(){
        for(var i = 0; i < this.bgs.length; i += 1){
            var bg = this.bgs[i];
            if(bg.x + bg.getBgSize().width + this.bgNode.x < -360
            || bg.x + this.bgNode.x > 1080){
                    this.outScreenBG = i;
                    return;
            }
        }
        this.outScreenBG = null;
    },
    // 获取另一个背景
    getOtherBg: function(){
        if(this.outScreenBG === 0){
            return this.bgs[1];
        }  else {
            return this.bgs[0];
        }
    },
    // 刷新地图
    refreshBgPosition: function(x){
        if(this.isBgOut){return;}
        this.findOutScreenBg();
        if(!this.outScreenBG){ return;}
       /* if(Math.abs(this.bgNode.y) >= this.bgs[0].getBgSize().height){  // 超过背景高度，地图不在移动
            this.bgs[0].setVisible(false);
            this.bgs[1].setVisible(false);
            this.isBgOut = true;
            return ;
        }*/
        var otherBg = this.getOtherBg();
        if(x < 0 && this.bgs[this.outScreenBG].x < otherBg.x && otherBg.x + this.bgNode.x + otherBg.getBgSize().width < otherBg.getBgSize().width - 100){  // 左移动
            this.bgs[this.outScreenBG].x = otherBg.x + otherBg.getBgSize().width;
        } else if(x > 0 && this.bgs[this.outScreenBG].x > otherBg.x && otherBg.x + this.bgNode.x + x > 0) {
            this.bgs[this.outScreenBG].x = otherBg.x - otherBg.getBgSize().width;
        }
    },
    //金币复活逻辑
    replayCoins:function(){
        this.nextFloor = null;
        this.curFloor = null;
        this.maxFloor = null;
        this.commands = [];

        Global.pause = false;

        var num = Global.curLevel%10;
        Global.curLevel = Global.curLevel-num;
        if(Global.curLevel<10){
            Global.curLevel=10;
        }

        Global.replayNum += 1;

        this.initData();
        this.backHomeNum = 2;
        this.isFirstTouch=false;

        for(var i = 0; i < GameLogic.mapNodes.length; i += 1){
            GameLogic.mapNodes[i].setVisible(false);
            GameLogic.mapNodes[i].num = 0;
        }
        this.bgNode.removeFromParent(true);
        this.obstacleNode.attr({x:88, y:300});

        this.role.removeFromParent(true);
        this.role = null;
        this.createRole();
        this.createBg();
        this.createSky();
        this.createEmitter();
        this.createObstacleNode();

        GameLogic.curFloor = this.maps[0].getFloorsNode().getChildByName("floor_" + 1);
        Global.curLevel = GameLogic.curFloor.getTag();
        this.startLevel = Global.curLevel;
        Event.emit(EVENT_ID.REFRESH_GOLD);
        //Event.emit(EVENT_ID.MENU_IN);
        Event.emit(EVENT_ID.JUMP);

        this.refreshFloorsArray();
        this.refreshWallsArray();
        this.disableOutScreen();
        this.scheduleUpdate();

    },
    // 重玩逻辑
    replay: function(){
        this.initData();
        this.backHomeNum = 2;

        for(var i = 0; i < GameLogic.mapNodes.length; i += 1){
            GameLogic.mapNodes[i].setVisible(false);
            GameLogic.mapNodes[i].num = 0;
        }
        this.bgNode.removeFromParent(true);
        this.obstacleNode.attr({x:88, y:300});

       this.role.removeFromParent(true);
       this.role = null;
       this.createRole();
        this.createBg();
        this.createSky();
        this.createEmitter();
        this.createObstacleNode();

        GameLogic.curFloor = this.maps[0].getFloorsNode().getChildByName("floor_" + 1);
        Global.curLevel = GameLogic.curFloor.getTag();
        this.startLevel = Global.curLevel;
        Event.emit(EVENT_ID.REFRESH_GOLD);
        Event.emit(EVENT_ID.MENU_IN);
        Event.emit(EVENT_ID.JUMP);

        this.refreshFloorsArray();
        this.refreshWallsArray();
        this.disableOutScreen();
        this.scheduleUpdate();
    },
    // 屏蔽屏幕外的物件
    disableOutScreen: function(){
        // if(Global.curLevel - Global.historyLevel <= 20 && type === 0){ return ;}
        var value = 0;
        // this.hitWalls = [];
        // this.hitFloors = [];
        for(var i = 0; i < this.walls.length; i += 1){
            var pos = cc.pAdd(this.walls[i].getPosition(), this.walls[i].getParent().getParent().getParent().getPosition());
            var size = this.walls[i].getContentSize();
            this.walls[i]["gameRect"].x = pos.x - size.width/2;
            this.walls[i]["gameRect"].y = pos.y - size.height/2;

            var y = this.obstacleNode.y + this.walls[i].gameRect.y;
            if(y >= 1588 || y < -108){
                this.walls[i].setVisible(false);
            } else {
                // this.hitWalls[value ++] = this.walls[i];
                this.walls[i].setVisible(true);
            }
        }
        var index = 0;
        for(var j = 0; j < this.floors.length; j += 1){
            var pos = cc.pAdd(this.floors[j].getPosition(), this.floors[j].getParent().getParent().getParent().getPosition());
            var size = this.floors[j].getContentSize();
            size.width = size.width * this.floors[j].getScaleX();

            this.floors[j]["gameRect"].x = pos.x - size.width/2;
            this.floors[j]["gameRect"].y = pos.y - size.height/2;

            var y = this.obstacleNode.y + this.floors[j].gameRect.y;
            if(y >= 1387 || y < -14){
                this.floors[j].setVisible(false);
                this.hideFloorDecorate(this.floors[j], false);
            }else{
                // this.hitFloors[index++] = this.floors[j];
                this.floors[j].setVisible(true);
                this.hideFloorDecorate(this.floors[j], true);
            }
        }
    },
    // 隐藏墙上的装饰物
    hideFloorDecorate: function(child, is){
        if(child.whiteWall){child.whiteWall.setVisible(is)};
        if(!child.decorates){ return;}
        for(var i = 0; i < child.decorates.length; i += 1){
            child.decorates[i].setVisible(is && gameUserData.eye === 1);
        }
    },
    // 回到起点
    backToHome: function(){
        if(Global.curLevel - this.startLevel < Global.resurgenceLevel){
            Global.pause = true;
            // Global.curLevel = 1;
            //this.backHomeNum -= 1;
            GameLogic.curFloor = this.maps[0].getFloorsNode().getChildByName("floor_" + 1);
            Global.curLevel = GameLogic.curFloor.getTag();
            var is = this.isFirstTouch;
            this.initData();
            this.isFirstTouch = is;
            // this.role.replayRole();
            this.role.setRoleState(ROLE_STATE.RUN);
            this.role.attr({x: 272, y:10});
            this.role.setVisible(false);
            // this.obstacleNode.y = 300;
            // this.bgNode.y = 0;
            this.isCanTouch = false;
            GameLogic.commands = [];
            Event.emit(EVENT_ID.JUMP);
            var t = Math.abs(this.obstacleNode.y/900);

            var sq1 = cc.sequence(cc.moveTo(t, cc.p(88,300)), cc.delayTime(0.02),cc.callFunc(function(){
                this.isCanTouch = true;
                Global.pause = false;
                this.role.setVisible(true);
                this.disableOutScreen();
            }.bind(this)));
            this.obstacleNode.runAction(sq1);

            var sq2 = cc.sequence(cc.moveTo(t, cc.p(0,0)));
            this.bgNode.runAction(sq2);

            this.recoveryMapState();

            return true;
        }
        return false;
    },
    // 恢复地图
    recoveryMapState: function(){
        for(var i = 0; i < Global.maps.length - 1; i += 1){
            this.maps[i].isMoveEnd = false;
            this.maps[i].recoveryDecorate();
        }
    },
    // 生成星星
    createStar: function(){
        var num = Utils.getRandomNum(3);
        for(var i = 0; i < num; i += 1){
            var y = Math.abs(this.bgNode.y) + 720 + Utils.getRandomNum(640);
            var x = this.bgNode.x * -1 + Utils.getRandomNum(720);
            var scale =  1 - Math.random() * 0.5;
            var index = Utils.getRandomNum(18);

            var star = this.getOutScreenObject(this.stars);
            if(star === null){
                star = new cc.Sprite("#bg/star" + index + ".png");
                this.bgNode.addChild(star);
                this.stars.push(star);
            }
            star.setPosition(cc.p(x, y));
            star.setScale(scale);
            star.setCascadeOpacityEnabled(true);
            star.setOpacity(0);
            star.runAction(cc.sequence(cc.fadeIn(2),cc.delayTime(0.3),cc.fadeOut(2)).repeatForever());
        }
    },
    // 随机云的出现
    randomCloudLevel: function(){
        for(var i = 0; i < 2; i += 1){
            var num = 109 + Utils.getRandomNum(91);
            // num = 15;
            this.clouds[i] = num;
        }
    },
    // 生成云
    createCloud: function(){
        for(var i = 0; i < this.clouds.length; i += 1){
            if(this.clouds[i] === Global.curLevel - Global.historyLevel){
                this.clouds.splice(i, 1);
                this.createRandomCloud();
            }
        }
    },
    // 随机生成云
    createRandomCloud: function(){
        var mapX = this.obstacleNode.x;
        var num = Utils.getRandomNum(3);
        for(var i = 0; i < num; i += 1){
            var randX = Utils.getRandomNum(150);
            var randY = Utils.getRandomNum(300);
            var cloud = new cc.Sprite("#bg/cloud.png");
            var size = cloud.getContentSize();

            var fact = Utils.getRandomNum(2) === 1? 1: -1;
            var x = 0 ,toX = 0;
            if(fact === 1){
                x = -this.obstacleNode.x - size.width - randX;
                toX = 720 + size.width + randX + this.obstacleNode.x;
            } else {
                x = 720 - this.obstacleNode.x + size.width + randX;
                toX = -720 - size.width - randX - this.obstacleNode.x;
            }
            var value = Utils.getRandomNum(4) + 6;
            cloud.runAction(cc.sequence(cc.moveBy(value, cc.p(toX,0)),cc.callFunc(function(){
                this.removeFromParent();
            }.bind(cloud))));
            cloud.x = x,cloud.y = GameLogic.curFloor.gameRect.y + 800 + randY;
            this.obstacleNode.addChild(cloud, GAME_TAG.ROLE);
        }
    },
    // 生成云
/*    createCloud: function(map, pos){
        if(Global.cloudLevel/10 > map.num){return;}
        var values = [1,2,3,4,5,6,7,8,9,10];
        var randomNum = Utils.getRandomNum(3);
        for(var i = 0; i < randomNum; i += 1){
            var num = Utils.getRandomNum(values.length) - 1;
            var floorNode = map.getFloorsNode();
            var floor = floorNode["floor_"+ values[num]];

            var cloud = this.getOutScreenObject(this.clouds);
            if(cloud === null){
                cloud = new cc.Sprite("#bg/cloud.png");
                this.obstacleNode.addChild(cloud, GAME_TAG.ROLE);
                this.clouds.push(cloud);
            }
            var width = floor.gameRect.width * floor.getScaleX();
            var x = width/4 + Utils.getRandomNum(width/2);
            cloud.x = floor.x - width/2 + x + pos.x, cloud.y = floor.y + 70 + pos.y;
            var fact = Utils.getRandomNum(2) === 1? 1: -1;
            var t = width/200;
            var sq = cc.sequence(cc.moveBy(t,cc.p(fact*width/2, 0)), cc.moveBy(t, cc.p(-1*fact*width/2, 0)));
            cloud.stopAllActions();
            cloud.runAction(sq.repeatForever());

            values.splice(num, 1);
        }
    },*/
    // 云和星星是否出屏幕外了
    getOutScreenObject: function(array){
        for(var i = 0 ;i < array.length; i += 1){
            var parent = array[i].getParent();
            var size = array[i].getContentSize();
            var pos = parent.convertToWorldSpace(array[i].getPosition());
            if(pos.x < -size.width/2 || pos.x - size.width/2 > 720 || pos.y < - size.height/2){
                return array[i];
            }
        }
        return null;
    },
    // 角色低于上次位置的时候检测一下
    roleIsUnderFloor: function(role, floor){
        if(role.y < floor.gameRect.y - 20){
            return true;
        }
        return false;
    }
});