// 暂停界面
var PauseLayer = cc.Layer.extend({
    mainUI: null,
    ctor: function(){
        this._super();
    },
    onEnter: function(){
        this._super();
        Global.pause = true;
        this.initUI();
        this.initListener();
        this.eyeState();
        this.soundState();
    },
    onExit: function(){
        this._super();
    },
    initUI: function(){
        var nodeRes = cc.loader.getRes(res.res_game_pause_layer_json);
        var data = nodeRes.Content.Content.ObjectData;
        this.mainUI = Utils.parseNodeToUI(data, 0);

        // this.mainUI = loadCSS(res.res_game_pause_layer_json);
        this.addChild(this.mainUI);
        this.mainUI.pauseNode.y -= 260;
        this.mainUI.settingNode.y -= 260;
        this.moveUp();
    },

    moveUp: function(){
        AnimationMgr.moveUpAndDown(this.mainUI.pauseNode, 260);
    },

    moveDown: function(){
        this.mainUI.pauseNode.runAction(cc.moveBy(0.1, cc.p(0, -260)));
        this.mainUI.settingNode.setVisible(true);
        this.mainUI.settingNode.runAction(cc.sequence(cc.delayTime(0.1), cc.moveBy(0.1, cc.p(0,260))));
    },
    initListener: function(){
        engine.gameEvent.createMouseUp(this.mainUI.pauseNode.btnGo, this.onGo.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.pauseNode.btnSet, this.onSet.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.pauseNode.btnHome, this.onHome.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.settingNode.btnSound, this.onSound.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.settingNode.btnEye, this.onEye.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.settingNode.btnClose, this.onClose.bind(this), buttonScalingType);
    },
    // 声音按钮状态
    soundState: function(){
        if(gameUserData.sound){
            engine.gameSound.openMusic();
            Event.emit(EVENT_ID.MUSIC);
            var spriteFrame1 = cc.spriteFrameCache.getSpriteFrame("setting/btn_sound.png");
            if (spriteFrame1){
                this.mainUI.settingNode.btnSound.initWithSpriteFrame(spriteFrame1);
            }
        } else {
            engine.gameSound.stopMusic();
            var spriteFrame2 = cc.spriteFrameCache.getSpriteFrame("setting/btn_close_sound.png");
            if (spriteFrame2){
                this.mainUI.settingNode.btnSound.initWithSpriteFrame(spriteFrame2);
            }
        }
    },
    // 眼睛状态
    eyeState: function(){
        if(gameUserData.eye){
            var spriteFrame1 = cc.spriteFrameCache.getSpriteFrame("setting/btn_eye.png");
            if (spriteFrame1){
                this.mainUI.settingNode.btnEye.initWithSpriteFrame(spriteFrame1);
            }
        } else {
            var spriteFrame2 = cc.spriteFrameCache.getSpriteFrame("setting/btn_colse_eye.png");
            if (spriteFrame2){
                this.mainUI.settingNode.btnEye.initWithSpriteFrame(spriteFrame2);
            }
        }
    },
    onGo: function(){
        SoundMgr.playEffect(soundRes.sound_btn);
        Event.emit(EVENT_ID.PAUSE_IN_OUT);
        Global.pause = false;
        this.removeFromParent();
    },
    onSet: function(){
        SoundMgr.playEffect(soundRes.sound_btn);
        this.moveDown();
    },
    onSound: function(){
        SoundMgr.playEffect(soundRes.sound_btn);
        gameUserData.sound = gameUserData.sound ? 0 : 1;
        SoundMgr.isOpen = gameUserData.sound === 1;
       /* if(gameUserData.sound === 0){
            SoundMgr.stopMusic();
            SoundMgr.stopAllEffects();
        }*/
        saveUserData();
        this.soundState();
    },
    onEye: function(){
        SoundMgr.playEffect(soundRes.sound_btn);
        gameUserData.eye = gameUserData.eye ? 0 : 1;
        this.eyeState();
        Event.emit(EVENT_ID.EYE);
    },
    onClose: function(){
        SoundMgr.playEffect(soundRes.sound_btn);
        this.mainUI.settingNode.runAction(cc.moveBy(0.1, cc.p(0, -260)));
        this.mainUI.pauseNode.runAction(cc.sequence(cc.delayTime(0.1), cc.moveBy(0.1, cc.p(0,260))));
    },
    onHome: function () {
        SoundMgr.playEffect(soundRes.sound_btn);

        GameLogic.reset();
        Global.historyLevel = 0;
        Global.replayNum = 0;


        Global.pause = false;
        this.removeFromParent();

        Event.emit(EVENT_ID.REPLAY);
    }
});
// 游戏界面
var GameMenuLayer = cc.Layer.extend({
    mainUI: null,
    ctor: function(){
        this._super();
    },
    onEnter: function(){
        this._super();
        this.initUI();
        this.initListener();
        this.initText();
        this.registerEvent();
        this.showTips();
    },
    registerEvent: function(){
        Event.register(EVENT_ID.PAUSE_IN_OUT, function(){
            this.mainUI.btnPause.runAction(cc.moveBy(0.2, cc.p(0, -300)));
        }, this);
        Event.register(EVENT_ID.START_GAME, function(data){
            this.logoOut();
        }, this);
        Event.register(EVENT_ID.MENU_IN, function(data){
            this.logoIn();
        }, this);

        Event.register(EVENT_ID.JUMP, function(data){
            this.initText();
        }, this);

        Event.register(EVENT_ID.TASK, function(data){
            this.showTips();
        }, this);
    },
    onExit: function(){
        this._super();
        Event.remove(EVENT_ID.PAUSE_IN_OUT, this);
        Event.remove(EVENT_ID.START_GAME, this);
        Event.remove(EVENT_ID.JUMP,this);
        Event.remove(EVENT_ID.MENU_IN, this);
        Event.remove(EVENT_ID.TASK, this);
    },
    initUI: function(){

        var nodeRes = cc.loader.getRes(res.res_game_game_menu_json);
        var data = nodeRes.Content.Content.ObjectData;
        this.mainUI = Utils.parseNodeToUI(data, 0);

        // this.mainUI = loadCSS(res.res_game_game_menu_json);
        this.addChild(this.mainUI);

        this.mainUI.logo.y += 385;
        this.mainUI.btnPause.y += 300;
        this.logoIn();

        this.mainUI.hand.runAction(AnimationMgr.getHandAction().repeatForever());
    },
    initListener: function(){
        engine.gameEvent.createMouseUp(this.mainUI.btnPause, this.onPause.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.btnRole, this.onRole.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.btnTask, this.onTask.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.btnShop, this.onShop.bind(this), buttonScalingType);

        engine.gameEvent.createMouseUp(this.mainUI.btnLead, this.onLead.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.btnArch, this.onArch.bind(this), buttonScalingType);
    },
    initText: function(){
        // cc.log("level = " + Global.curLevel);
        this.mainUI.levelText.setString(Global.curLevel);
    },
    // 最高分in
    maxScoreIn: function(){
        AnimationMgr.moveLeftAndRight(this.mainUI.scoreNode, 210);
    },
    maxScoreOut: function(){
        AnimationMgr.moveLeftAndRight(this.mainUI.scoreNode, -210);
    },
    // 暂停in
    pauseIn: function(){
        AnimationMgr.moveUpAndDown(this.mainUI.btnPause, -140);
    },
    pauseOut: function(){
        AnimationMgr.moveUpAndDown(this.mainUI.btnPause, 140);
    },
    // logo  in
    logoIn: function(){
        this.showTips();
        //this.mainUI.logo.y = 1485;
        this.mainUI.logo.y = 1600;
        this.mainUI.btnPause.y = 1493;
        AnimationMgr.moveUpAndDown(this.mainUI.logo, -500);
        this.mainUI.btnRole.y = 110;
        this.mainUI.btnTask.y = 110;
        this.mainUI.btnShop.y = 110;
        this.mainUI.btnLead.y = 110;
        this.mainUI.btnArch.y = 110;
        this.mainUI.hand.setVisible(true);
    },
    logoOut: function(){
        this.mainUI.tips.setVisible(false);
        AnimationMgr.moveUpAndDown(this.mainUI.logo, 500);
        AnimationMgr.moveUpAndDown(this.mainUI.btnRole, -175);
        AnimationMgr.moveUpAndDown(this.mainUI.btnTask, -175);
        AnimationMgr.moveUpAndDown(this.mainUI.btnShop, -175);
        AnimationMgr.moveUpAndDown(this.mainUI.btnLead, -175);
        AnimationMgr.moveUpAndDown(this.mainUI.btnArch, -175);
        // this.mainUI.hand.stopAllActions();
        this.mainUI.hand.setVisible(false);
    },
    onLead: function(){
        cc.log("on onShop !!!");
        SoundMgr.playEffect(soundRes.sound_btn);
        googleShowLead();
    },
    onArch: function(){
        cc.log("on onShop !!!");
        SoundMgr.playEffect(soundRes.sound_btn);
        googleShowArch();
    },
    onShop: function(){
        cc.log("on onShop !!!");
        SoundMgr.playEffect(soundRes.sound_btn);
        var layer = new ShopLayer();
        createMaskLayer(layer);
        this.addChild(layer);
    },
    onPause: function(){
        cc.log("on pause !!!");
        SoundMgr.playEffect(soundRes.sound_btn);
        this.mainUI.btnPause.runAction(cc.moveBy(0.2, cc.p(0, 300)));
        this.getParent().createPauseLayer();
    },
    onRole: function(){
        cc.log("on role !!!");
        SoundMgr.playEffect(soundRes.sound_btn);
        Global.historyLevel = 0;
        Global.replayNum = 0;
        GameLogic.reset();

        var layer = new ChooseLayer();
        Global.pause = true;
        createMaskLayer(layer);
        this.getParent().addChild(layer, 1001);
        // transitions(ChooseScene);
    },
    onTask: function(){
        cc.log("on task !!!");
        SoundMgr.playEffect(soundRes.sound_btn);
        var layer = new TaskLayer();
        createMaskLayer(layer);
        this.addChild(layer);
    },
    // 是否满足条件
    isCompleted: function(data){
        switch(data.type){
            case 1:
                return gameUserData.maxLevel >= data.condition;  // 最大层数
            case 2:
                return gameUserData.roles.length >= data.condition; // 角色数量
            case 3:
                return gameUserData.scenes.length >= data.condition; // 角色数量
                break;
            default:
                cc.log("is default !!!!!!");
                return false;
                break;
        }
    },
    // 完成任务的状态 0 不能领取 1 可以领取 2 已经领取
    getTaskState: function(data){
        if(gameUserData.tasks[data.id - 1]){ return 2;}
        var bool = this.isCompleted(data);

        if(bool){
            return 1;
        } else {
            return 0;
        }
    },
    showTips: function(){
        this.mainUI.tips.setVisible(false);
        for(var i = 1; i <= TaskConfig.length; ++i){
            var num = this.getTaskState(TaskConfig[i - 1]);
            if(num === 1){
                this.mainUI.tips.setVisible(true);
                return;
            }
        }
    }
});

// 设置界面
var SettingLayer = cc.Layer.extend({
    ctor: function(){
        this._super();
    },
    onEnter: function(){
        this._super();
    },
    onExit: function(){
        this._super();
    }
});
// 结算界面
var ResultLayer = cc.Layer.extend({
    mainUI: null,
    sceneID: 0,
    scene: null,
    light: null,
    letter: null,
    maskLayer: null,
    canTouch: true,
    unlockID: 0, // 解锁的id
    letterSprite: null,
    ctor: function(id){
        this._super();
        this.sceneID = id || gameUserData.curScene;
        this.unlockID = 0;
    },
    onEnter: function(){
        this._super();
        SoundMgr.playEffect(soundRes.sound_dialog);
        this.initUI();
        this.unlockScene();
    },
    onExit: function(){
        this._super();
    },
    // 场景动画
    sceneActon: function(id){
        var data = Utils.getDataByID(GameSceneConfig, id);
        var sprite = new cc.Sprite("#" + data.icon);
        sprite.setPosition(cc.p(360, 150));
        sprite.setScale(0.65);
        sprite.setVisible(false);
        this.addChild(sprite, 4);
        this.scene = sprite;
        sprite.runAction(cc.sequence(cc.rotateTo(0.4,5), cc.rotateTo(0.4, -5)).repeatForever());
    },
    // 信封动画
    letterAction: function(){
        SoundMgr.playEffect(soundRes.sound_open_letter);
        var light = createGameSkeletalAnimation(res.res_game_letter_action_json, "light", 30, "pngdata/letter");
        light.setPosition(cc.p(360, 150));
        light.setScale(2);
        light.setVisible(false);
        this.addChild(light,2);

        var letter = createGameSkeletalAnimation(res.res_game_letter_action_json, "letter", 30, "pngdata/letter");
        letter.playCount = 1;
        letter.isRemoveFromComplete = true;
        letter.frameFun = function(frameIndex){
            if(frameIndex === 15){
                light.setVisible(true);
                if(this.scene !== null){
                    this.canTouch = true;
                    this.scene.setVisible(true);
                }
            }
        }.bind(this);
        letter.setPosition(cc.p(360, 150));
        this.addChild(letter, 3);
        this.letter = letter;
        this.light = light;
    },
    // 解锁场景
    unlockScene: function(){
        this.mainUI.tipsNode.setVisible(false);
        var data = UnlockSceneCondition[this.sceneID - 2001];
        if(!data){ return;}
        var is = GameLogic.isInArray(gameUserData.scenes,data.unlockID);
        if(!is && Global.curLevel >= data.floor){
            this.unlockID = data.unlockID;
            this.canTouch = false;
           this.scheduleOnce(function(){
                this.sceneActon(this.unlockID);
                this.letterAction();
           }.bind(this),0.5);

            gameUserData.scenes.push(data.unlockID);
            saveUserData();
        }else if(!is){
            var language = Global.language;
            this.mainUI.tipsNode.setVisible(true);
            var sceneData = Utils.getDataByID(GameSceneConfig, data.unlockID);
            Utils.changeTexture(this.mainUI.tipsNode.icon, sceneData.icon);
            this.mainUI.tipsNode.text.setString(Global.unlockNextSceneText.pre[language] + data.floor + Global.unlockNextSceneText.middle[language]);
            this.mainUI.tipsNode.name.setString(sceneData[language]);
            this.mainUI.tipsNode.target.setString(data.floor);
        }

    },
    initUI: function(){
        // this.mainUI = loadCSS(res.res_game_result_json);
        var nodeRes = cc.loader.getRes(res.res_game_result_json);
        var data = nodeRes.Content.Content.ObjectData;
        this.mainUI = Utils.parseNodeToUI(data, 0);

        this.addChild(this.mainUI,1);

        if(gameUserData.shareCount<=0){
            this.mainUI.shareCoins.setVisible(false);
        }

        this.refreshText();

        gameUserData.maxLevel  = gameUserData.maxLevel > Global.curLevel ? gameUserData.maxLevel : Global.curLevel;
        saveUserData();
        engine.gameEvent.createMouseUp(this.mainUI.btnGo, this.onContinue.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.btnShare, this.onFaceBookShare.bind(this), buttonScalingType);
    },
    refreshText: function(){
        // this.mainUI.text.setString(Global.curLevel);
        AnimationMgr.scoreAction(this.mainUI.text, 0, Global.curLevel, 8);
        // this.mainUI.goldText.setString(Global.curGold);
        AnimationMgr.scoreAction(this.mainUI.goldText, 0, Global.curGold, 4);
    },
    onFaceBookShare: function(){
        SoundMgr.playEffect(soundRes.sound_btn);
        facebookShare();

    },
    onContinue: function(){
        if(!this.canTouch){return;}
        this.removeFromParent();
        SoundMgr.playEffect(soundRes.sound_btn);
        GameLogic.reset();
        //if(Global.historyLevel >= 100){
        //    var num = Global.historyLevel%10;
        //    Global.curLevel = Math.ceil((Global.historyLevel - num)/20) * 10;
        //    Global.replayNum += 1;
        //}
        if(this.unlockID > 0){
            gameUserData.curScene = this.unlockID;
        }

        // transitions(HomeScene);
        Event.emit(EVENT_ID.REPLAY);
        Event.emit(EVENT_ID.TASK);
    }
});
// 左上角的layer
var LeftAndUPLayer = cc.Layer.extend({
    mainUI: null,
    ctor: function(){
        this._super();
    },
    onEnter: function(){
        this._super();
        this.initUI();
        this.initText();
        this.goldAction();
        Event.register(EVENT_ID.REFRESH_GOLD, function(type){
            this.initText(type);
        }, this);
    },
    onExit: function(){
        this._super();
        Event.remove(EVENT_ID.REFRESH_GOLD, this);
    },
    // 金币动画
    goldAction: function(){
        this.mainUI.gold.runAction(AnimationMgr.getGoldRotateAction(1/10).repeatForever());
    },
    initUI: function(){
        // this.mainUI = loadCSS(res.res_game_lef_up_json);
        var nodeRes = cc.loader.getRes(res.res_game_lef_up_json);
        var data = nodeRes.Content.Content.ObjectData;
        this.mainUI = Utils.parseNodeToUI(data, 0);
        this.addChild(this.mainUI);
    },
    initText: function(type){
        this.mainUI.goldText.setString(gameUserData.gold);
        if(type !== 1){
            var level = gameUserData.maxLevel > Global.curLevel ? gameUserData.maxLevel : Global.curLevel;
            this.mainUI.maxText.setString(level);
        }
    }
});
// 复活界面
var ResurgenceLayer = cc.Layer.extend({
    mainUI: null,
    num: 0,
    ctor: function(){
        this._super();
    },
    onEnter: function(){
        this._super();
        SoundMgr.playEffect(soundRes.sound_dialog);
        // this.mainUI = loadCSS(res.res_game_resurgence_json);

        var nodeRes = cc.loader.getRes(res.res_game_resurgence_json);
        var data = nodeRes.Content.Content.ObjectData;
        this.mainUI = Utils.parseNodeToUI(data, 0);

        this.addChild(this.mainUI);
        this.initText();

        engine.gameEvent.createMouseUp(this.mainUI.btnOk, this.onOk.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.btnNo, this.onNo.bind(this), buttonScalingType);
    },
    onExit: function(){
        this._super();
    },
    initText: function(){
        var value = Global.historyLevel%10;
        Global.curLevel = Math.ceil((Global.historyLevel - value)/20) * 10 + 1;
        this.mainUI.text.setString(Global.curLevel);

        var num = Global.replayNum * 10 + 20;
        num = num >= 90? 90: num;
        this.mainUI.money.setString(num);
    },
    onOk: function(){
        cc.log("on ok !!!");
        SoundMgr.playEffect(soundRes.sound_btn);

        GameLogic.reset();

        var money = Global.replayNum * 10 + 20;
        money = money >= 90? 90: money;
        if(gameUserData.gold >= money){
            gameUserData.gold -= money;
            saveUserData();

            var num = Global.historyLevel%10;
            Global.curLevel = Math.ceil((Global.historyLevel - num)/20) * 10 + 1;
            Global.replayNum += 1;

            // transitions(HomeScene);
            this.removeFromParent();
            Event.emit(EVENT_ID.REPLAY);
        } else {
            cc.log("not rich enough !!!");
        }
    },
    onNo: function(){
        SoundMgr.playEffect(soundRes.sound_btn);

        GameLogic.reset();
        Global.historyLevel = 0;
        Global.replayNum = 0;
        // transitions(HomeScene);
        this.removeFromParent();
        Event.emit(EVENT_ID.REPLAY);
    }
});
// 获取信封
var LetterLayer = cc.Layer.extend({
    mainUI: null,
    isTouch: false,
    role:null,
    scene:null,
    ctor: function(){
        this._super();
        this.role = null;
        this.scene = null;
    },
    onEnter: function(){
        this._super();
        SoundMgr.playEffect(soundRes.sound_dialog);

        var nodeRes = cc.loader.getRes(res.res_game_letter_json);
        var data = nodeRes.Content.Content.ObjectData;
        this.mainUI = Utils.parseNodeToUI(data, 0);
        this.addChild(this.mainUI);
        this.initText();

        engine.gameEvent.createMouseUp(this.mainUI.btnOk, this.onOk.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.btnNo, this.onNo.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this, this.onTouch.bind(this));
    },
    // 信封动画
    letterAction: function(){
        this.scheduleOnce(function(){
            SoundMgr.playEffect(soundRes.sound_open_letter);
        },0.2);

        var light = createGameSkeletalAnimation(res.res_game_letter_action_json, "light", 30, "pngdata/letter");
        light.setPosition(cc.p(360, 640));
        light.setScale(2);
        light.setVisible(false);
        this.addChild(light);

        var letter = createGameSkeletalAnimation(res.res_game_letter_action_json, "letter", 30, "pngdata/letter");
        letter.playCount = 1;
        letter.isRemoveFromComplete = true;
        letter.frameFun = function(frameIndex){
            if(frameIndex === 15){
                light.setVisible(true);
                if(this.role !== null){
                    this.role.setVisible(true);
                }
                if(this.scene !== null){
                    this.scene.setVisible(true);
                }
            }
        }.bind(this);
        letter.setPosition(cc.p(360, 640));
        this.addChild(letter, 1000);
    },
    // 角色动画
    roleAction: function(id){
        this.role = new Role(id);
        this.role.setScale(2);
        this.role.setVisible(false);
        this.addChild(this.role, 1);
        this.role.setPosition(cc.p(365 - this.role.size.width, 590 - this.role.height));
    },
    // 场景动画
    sceneActon: function(id){
        var data = Utils.getDataByID(GameSceneConfig, id);
        var sprite = new cc.Sprite("#" + data.icon);
        sprite.setPosition(cc.p(360, 640));
        sprite.setScale(0.65);
        sprite.setVisible(false);
        this.addChild(sprite, 1);
        this.scene = sprite;
        sprite.runAction(cc.sequence(cc.rotateTo(0.4,5), cc.rotateTo(0.4, -5)).repeatForever());
    },
    // 开奖
    lottery: function(){
        if(gameUserData.lotterys.length < 1){ return;}
        // var index = Utils.getRandomNum(gameUserData.lotterys.length) - 1;
        // var id = gameUserData.lotterys[index];
        // Utils.removeDataFromArray(gameUserData.lotterys, id);
        var id = gameUserData.lotterys.shift();
        if(id > 2000){
            this.sceneActon(id);
            gameUserData.scenes.push(id);
        } else {
            this.roleAction(id);
            gameUserData.roles.push(id);
        }
        saveUserData();
    },
    onExit: function(){
        this._super();
    },
    initText: function(){
        this.mainUI.text.setString(Global.messagePrice);
    },
    onOk: function(){
        // if(!this.isTouch){return;}
        SoundMgr.playEffect(soundRes.sound_btn);
        this.mainUI.setVisible(false);
        this.letterAction();
        this.lottery();

        gameUserData.gold -= Global.messagePrice;
        this.isTouch = false;
        this.scheduleOnce(function(){
            this.isTouch = true;
        }.bind(this), 1.5);

    },
    onNo: function(){
        cc.log("on no !!!");
        SoundMgr.playEffect(soundRes.sound_btn);
        this.removeFromParent();
        Event.emit(EVENT_ID.CHECK_RESULT);
    },
    onTouch: function(){
        cc.log("on touch !!!");
        // SoundMgr.playEffect(soundRes.sound_btn);
        if(!this.isTouch){return ;}
        this.onNo();
    }
});

// 购买金币不足
var LakeConinsLayer = cc.Layer.extend({
    mainUI: null,
    isTouch: false,
    ctor: function(){
        this._super();
    },
    onEnter: function(){
        this._super();
        SoundMgr.playEffect(soundRes.sound_dialog);

        var nodeRes = cc.loader.getRes(res.res_game_lakecoins_json);
        var data = nodeRes.Content.Content.ObjectData;
        this.mainUI = Utils.parseNodeToUI(data, 0);
        this.addChild(this.mainUI);
        engine.gameEvent.createMouseUp(this.mainUI.btnOk, this.onOk.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.btnNo, this.onNo.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this, this.onTouch.bind(this));
    },
    onExit: function(){
        this._super();
    },
    onOk: function(){
        SoundMgr.playEffect(soundRes.sound_btn);
        var layer = new ShopLayer();
        createMaskLayer(layer);
        this.getParent().addChild(layer,2002);
        this.removeFromParent();
    },
    onNo: function(){
        cc.log("on no !!!");
        SoundMgr.playEffect(soundRes.sound_btn);
        this.removeFromParent();
    },
    onTouch: function(){
        cc.log("on touch !!!");
        // SoundMgr.playEffect(soundRes.sound_btn);
        if(!this.isTouch){return ;}
        this.onNo();
    }
});

// 继续重玩
var ContiniuplayLayer = cc.Layer.extend({
    mainUI: null,
    isTouch: false,
    timeCount:3,
    isVedioType:true,
    ctor: function(){
        this._super();
        this.timeCount=3;
    },
    onEnter: function(){
        this._super();
        SoundMgr.playEffect(soundRes.sound_dialog);

        this.isVedioType=true;
        this.schedule(this.timeDelayMiss,1,3,1);
        var nodeRes = cc.loader.getRes(res.res_game_conplay_json);
        var data = nodeRes.Content.Content.ObjectData;
        this.mainUI = Utils.parseNodeToUI(data, 0);
        this.addChild(this.mainUI);

        var hasAd=hascontinuePlayVe();
        if(!hasAd&&Global.rePlayCount==1){
            Global.rePlayCount++;
        }

        this.mainUI.conTime.setString(""+this.timeCount);
        this.mainUI.money.setString(""+(Global.rePlayCount-1)*20);
        engine.gameEvent.createMouseUp(this.mainUI.btnOk, this.onOk.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this.mainUI.btnNo, this.onNo.bind(this), buttonScalingType);
        engine.gameEvent.createMouseUp(this, this.onTouch.bind(this));


        if(Global.rePlayCount==1){
            this.mainUI.btnNo.setVisible(false);
            this.mainUI.money.setVisible(false);
            this.isVedioType=true;
        }else{
            this.isVedioType=false;
            this.mainUI.btnOk.setVisible(false);
        }
    },
    timeDelayMiss:function(){
        this.timeCount--;
        this.mainUI.conTime.setString(""+this.timeCount);
        if(this.timeCount<0){
            this.getParent().addResultUITwo();
            this.unscheduleAllCallbacks();
            this.removeFromParent();
        }
    },
    onExit: function(){
        this._super();
    },
    onOk: function(){
        if( !this.isVedioType){
            return;
        }

        SoundMgr.playEffect(soundRes.sound_btn);

        if(!continuePlayVe(101)){
            Event.emit(EVENT_ID.CHECK_RESULT);
            this.removeFromParent();
            return;
        }
        Global.rePlayCount++;
        this.removeFromParent();



        //Event.emit(EVENT_ID.REPLAYCOINS);

    },
    onNo: function(){
        if( this.isVedioType){
            return;
        }
        SoundMgr.playEffect(soundRes.sound_btn);
        var money = (Global.rePlayCount-1)*20;
        if(gameUserData.gold >= money){
            gameUserData.gold -= money;
            saveUserData();
            Global.rePlayCount++;
            Event.emit(EVENT_ID.REFRESH_GOLD,1);
            Event.emit(EVENT_ID.REPLAYCOINS);
            this.removeFromParent();

        }else{

            //Global.rePlayCount++;
            //this.removeFromParent();
            //
            //Event.emit(EVENT_ID.REPLAYCOINS);

            toastNotCoins();
            cc.log("coins bu gou");
        }

    },
    onTouch: function(){
        cc.log("on touch !!!");
    }
});
