var TAG = {
    ITEM: 100
};

var TaskLayer = cc.Layer.extend({
    mainUI: null,
    language: null,
    ctor: function(){
        this._super();
        this.language = Global.language;
    },
    onEnter: function(){
        this._super();
        this.initUI();
        this.initText();
        this.initListener();
    },
    onExit: function(){this._super();},
    // 初始化界面
    initUI: function(){
        this.mainUI = loadCSS(res.res_game_task_layer_json);
        this.addChild(this.mainUI);

        this.createListView();
    },
    // 监听消息
    initListener: function(){
        engine.gameEvent.createMouseUp(this.mainUI.btnHome, this.onHome.bind(this), buttonScalingType);
    },
    // 刷新文本
    initText: function(){
        this.mainUI.goldText.setString(gameUserData.gold );
    },
    // 创建滑动框
    createListView: function(){
        var listView = new ccui.ListView();
        listView.setDirection(ccui.ScrollView.DIR_VERTICAL);
        listView.setContentSize(cc.size(640, 950));
        listView.setTouchEnabled(true);
        listView.setBounceEnabled(true);
        listView.x = 60;
        listView.y = 160;

        for(var i = 1; i <= TaskConfig.length; ++i){
            var item = this.createItem(i);
            listView.addChild(item);
        }
        this.addChild(listView);
    },
    // 完成任务的状态 0 不能领取 1 可以领取 2 已经领取
    getTaskState: function(data){
        if(gameUserData.tasks[data.id - 1]){ return 2;}
        var bool = this.isCompleted(data);

        if(bool){
            return 1;
        } else {
            return 0;
        }
    },
    // 是否满足条件
    isCompleted: function(data){
        switch(data.type){
            case 1:
                return gameUserData.maxLevel >= data.condition;  // 最大层数
            case 2:
                return gameUserData.roles.length >= data.condition; // 角色数量
            case 3:
                return gameUserData.scenes.length >= data.condition; // 角色数量
                break;
            default:
                cc.log("is default !!!!!!");
                return false;
                break;
        }
    },
    // 获取图片数据
    getImageData: function(id){
        var data = {};
        data.button_img = null;
        data.frame_img = null;
        if(id === 0){  // 不可以领取
            data.button_img = "task/task_no_touch.png";
            data.frame_img = "#task/task_frame_bg.png";
        } else if(id === 1){ // 可以领取
            data.button_img = "task/task_can_touch.png";
            data.frame_img = "#task/task_frame_bg.png";
        } else if(id === 2){ // 已经领取
            data.button_img = "task/task_already_get.png";
            data.frame_img = "#task/task_alredy_get_frame.png";
        }
        return data;
    },
    // 创建滑动框node
    createItem: function(index){
        var id = this.getTaskState(TaskConfig[index - 1]);
        var data = this.getImageData(id);

        var item = new ccui.Layout();
        item.setContentSize(640, 130);

        item.frame = new cc.Sprite(data.frame_img);
        item.frame.attr({x: 311, y: 61});
        item.addChild(item.frame);

        item.button = new ccui.Button(data.button_img, "", "", ccui.Widget.PLIST_TEXTURE);
        item.button.setTag(TAG.ITEM + index);
        item.button.setTouchEnabled(id === 1);
        item.button.setBright(false);
        item.button.addTouchEventListener(this.onTouch, this);
        item.button.attr({x: 515, y: 61});
        item.addChild(item.button);

        var text = new cc.LabelTTF(TaskConfig[index - 1][this.language],"Arial", 36, cc.size(370, 80),  cc.TEXT_ALIGNMENT_CENTER, cc.VERTICAL_TEXT_ALIGNMENT_CENTER);
        text.attr({x: 221, y: 61});
        item.addChild(text);

        if(id !== 2){
            item.gold = new cc.LabelTTF(TaskConfig[index - 1].reward, "Arial", 30, cc.size(100, 30),  cc.TEXT_ALIGNMENT_CENTER, cc.VERTICAL_TEXT_ALIGNMENT_CENTER);
            item.gold.attr({x: 495, y: 61});
            item.addChild(item.gold);

            item.goldIcon = new cc.Sprite("#task/task_gold.png");
            item.goldIcon.attr({x: 560, y: 61});
            item.addChild(item.goldIcon);
        } else {
            var ok = new cc.Sprite("#task/task_ok.png");
            ok.attr({x: 520, y: 72});
            item.addChild(ok);
        }

        if(id === 1){
            item.tips = new cc.Sprite("#home/tips.png");
            item.tips.attr({x: 570, y: 95});
            item.addChild(item.tips);
        }

        return item;
    },
    // 领取奖励 刷新item
    refreshItem: function(item){
        item.gold.setVisible(false);
        item.goldIcon.setVisible(false);
        var spriteFrame = cc.spriteFrameCache.getSpriteFrame("task/task_alredy_get_frame.png");
        if (spriteFrame){
            item.frame.initWithSpriteFrame(spriteFrame);
        }
        item.button.loadTextureNormal("task/task_already_get.png", ccui.Widget.PLIST_TEXTURE);
        item.button.setTouchEnabled(false);
        var ok = new cc.Sprite("#task/task_ok.png");
        ok.attr({x: 520, y: 72});
        item.addChild(ok);

        item.tips.setVisible(false);
    },
    onTouch: function(sender, type) {
        switch (type) {
            case ccui.Widget.TOUCH_BEGAN:
                break;
            case ccui.Widget.TOUCH_MOVED:
                break;
            case ccui.Widget.TOUCH_ENDED:
                SoundMgr.playEffect(soundRes.sound_btn);

                var tag = sender.getTag();

                gameUserData.tasks[tag - TAG.ITEM - 1] = 1;
                gameUserData.gold += TaskConfig[tag - TAG.ITEM - 1].reward;

                this.initText();
                this.refreshItem(sender.getParent());
                Event.emit(EVENT_ID.TASK);
                saveUserData();
                break;
            case ccui.Widget.TOUCH_CANCELED:
                break;
            default:
                break;
        }
    },
    onHome: function(){
        cc.log("on home !!!");
        SoundMgr.playEffect(soundRes.sound_btn);
        this.removeFromParent();
    }
});
