var AnimationMgr = {
    // 获取帧动画
    getActionByArray: function (array, fps) {
        var frames = [];
        for (var i = 0; i < array.length; i += 1) {
            var spriteFrame = cc.spriteFrameCache.getSpriteFrame(array[i]);
            if (spriteFrame) {
                frames.push(spriteFrame);
            } else {
                var tex = cc.textureCache.addImage(array[i]);
                if(!tex){
                    continue;
                }
                var rect = cc.rect(0, 0, 0, 0);
                rect.width = tex.width;
                rect.height = tex.height;
                spriteFrame = new cc.SpriteFrame(tex, rect);
                frames.push(spriteFrame);
            }
        }
        fps = fps || 1/24;
        var animation = new cc.Animation(frames, fps);
        var action = cc.animate(animation);
        return action;
    },
    // 获取动画数组
    getAnimationArray: function (name, num) {
        var animationArray = [];
        for (var i = 1; i <= num; i += 1) {
            animationArray.push(name + i + ".png");
        }
        return animationArray;
    },
    // 手指动画
    getHandAction: function () {
        return this.getActionByArray(this.getAnimationArray("home/hand", 2), 0.5);
    },
    // 上下移动动画
    moveUpAndDown: function(node, height){
        node.stopAllActions();
        node.runAction(cc.sequence(cc.moveTo(0.2, cc.p(node.x, node.y + height))));
    },
    // 左右移动
    moveLeftAndRight: function(node, weight){
        node.stopAllActions();
        node.runAction(cc.sequence(cc.moveTo(0.2, cc.p(node.x + weight, node.y))));
    },
    // 队长跑动动画
    getCaptainRoleRunAction: function(){
        return this.getActionByArray(this.getAnimationArray("captain", 12));
    },
    // 忍者跑动动画
    getNinjaRoleRunAction: function(){
        return this.getActionByArray(this.getAnimationArray("ninja", 12));
    },
    // 公主跑动动画
    getPrincessRoleRunAction: function(){
        return this.getActionByArray(this.getAnimationArray("princess", 12));
    },
    // 上班跑动动画
    getWorkerRoleRunAction: function(){
        return this.getActionByArray(this.getAnimationArray("worker", 12));
    },
    // 骷髅跑动动画
    getSkeletonRoleRunAction: function(){
        return this.getActionByArray(this.getAnimationArray("skeleton", 12));
    },
    // 机器跑动动画
    getRobotRoleRunAction: function(){
        return this.getActionByArray(this.getAnimationArray("robot", 12));
    },
    // 宇航员跑动动画
    getSpacemanRoleRunAction: function(){
        return this.getActionByArray(this.getAnimationArray("spaceman", 12));
    },
    // 金币旋转动画
    getGoldRotateAction: function(fps){
        return this.getActionByArray(this.getAnimationArray("gold", 8),fps);
    },
    // 警告
    getWarningAction: function(fps){
        return this.getActionByArray(this.getAnimationArray("bg/warning", 5),fps);
    },
/* ui 需要跑马的label
 * from  表示从多少分数开始
 * to    结束的分数
 * count 循环的次数
 **/
    scoreAction: function (ui, from, to, count) {
        var addT = 0;
        if (to - from >= count) {
            addT = parseInt((to - from) / count);
        } else {
            addT = 1;
        }
        ui.schedule(function(){
            from += addT;
            if (from >= to) {
                from = to;
                ui.setString(from);
                ui.unschedule(arguments.callee.caller);
            } else {
                ui.setString(from);
            }
        }.bind(this), 0.05);
    },
    // 雪花飘舞
    snowDance: function(){
        var _emitter = new cc.ParticleSnow();
        _emitter.initWithTotalParticles(300);
        _emitter.life = 8;
        _emitter.lifeVar = 1;

        // gravity
        _emitter.gravity = cc.p(0, -10);

        // speed of particles
        _emitter.speed = 100;
        _emitter.speedVar = 30;

        var startColor = _emitter.startColor;
        startColor.r = 230;
        startColor.g = 230;
        startColor.b = 230;
        _emitter.startColor = startColor;

        var startColorVar = _emitter.startColorVar;
        startColorVar.b = 26;
        _emitter.startColorVar = startColorVar;

        _emitter.emissionRate = _emitter.totalParticles / _emitter.life;

        _emitter.texture = cc.textureCache.addImage("res/bg/snow.png");
        _emitter.shapeType = cc.ParticleSystem.STAR_SHAPE;
        // _emitter.initWithTotalParticles(500);

        // this.setEmitterPosition();
        return _emitter;
    },
    // 雨水滑落
    rainDrop: function(){
        var _emitter = new cc.ParticleRain();
        _emitter.life = 7;
        _emitter.speed = 300;
        _emitter.texture = cc.textureCache.addImage("res/bg/rain.png");
        _emitter.shapeType = cc.ParticleSystem.BALL_SHAPE;
        return _emitter;
    }
};
