var EVENT_ID = {
    PAUSE_IN_OUT: 1,  // 暂停按钮进入
    START_GAME: 2,     // 开始游戏
    JUMP: 3,           // 跳跃
    REFRESH_GOLD: 4,    // 刷新金币
    REFRESH_FLOOR: 5,    // 刷新楼层,
    EYE: 6,               // 眼睛消息
    CHECK_RESULT: 7,       // 检测结果
    REPLAY: 8,              // 重玩
    MENU_IN:9,            // 进入菜单
    MUSIC:10,              // 背景音乐
    TASK:11,                // 刷新任务
    REPLAYCOINS:12,
    GOOGLEPAYVEDIO:13,
    CHOOSEREFRESHCOINS:14
};



var Event = {
    events: {},
    /*  注册消息
     *   event_id : string  注册的事件ID
     *   func: function 回调接口
     *   self: object 注册的类
     * */
    register: function (event_id, func, self) {
        if (!event_id || !func) {
            cc.log("register error param !!!");
            return;
        }
        if (this.checkIsExist(event_id, self)) {
            cc.log("already exist in event!!!");
            return;
        }
        var obj = {};
        obj.self = self || null;
        obj.func = func || null;

        if (!this.events[event_id]) {
            this.events[event_id] = [];
        }
        this.events[event_id].push(obj);
    },
    /*  检测是否已经注册
     *   event_id : string  注册的事件ID
     *   self: object 注册的类
     * */
    checkIsExist: function (event_id, self) {
        var events = this.events[event_id];
        if (!events) {
            return false;
        }
        for (var i = 0; i < events.length; ++i) {
            if (_.isEqual(events[i].self, self)) {
                return true;
            }
        }
        return false;
    },
    /*  发射消息
     *   event_id : string  注册的事件ID
     *   data: 数据
     * */
    emit: function (event_id, data) {
        var events = this.events[event_id];
        if (!events) {
            cc.log("event num is out !!!");
            return;
        }
        for (var i = 0; i < events.length; ++i) {
            var obj = events[i];
            this.callBack(obj, data || null)
        }
    },
    /* 广播消息
     *  data: 数据
     *  */
    broad: function (data) {
        for (var i in this.events) {
            for (var j = 0; j < this.events[i].length; ++j) {
                var obj = this.events[i][j];
                this.callBack(obj, data);
            }
        }
    },
    // 回调
    callBack: function (obj, data) {
        var self = obj.self;
        var func = obj.func;
        if (self && self.func) {
            self.func(data);
        } else {
            func.call(self, data);
            //func(data);
        }
    },
    // 移除注册的消息
    remove: function (event_id, self) {
        var events = this.events[event_id];
        if(!events){
            return;
        }
        for (var i = 0; i < events.length; ++i) {
            if (_.isEqual(events[i].self, self)) {
                events.splice(i, 1);
            }
        }
    },
    // 清理事件
    clean: function () {
        this.events = {};
    }
};