var GameLogic = {
    nextFloor: null, // 上一个地板
    curFloor: null,    // 当前脚下的底板
    maxFloor: null,    // 跳的最高的地板
    commands: [],     // 命令集合
    mapNodes: [],         // 地图集合
    beforeType: 0,        // 上一个随机生成的类型
    randNum: 0,
    hDistance: 0,        // 横向移动距离
    // 检测所有墙
    checkRoleAndWalls: function(role, node){
        if(!role || !node) return;
        var children = node;

        if(children && children.length <= 1){
            cc.log("children is not a array !!!");
            return false;
        }

        for(var i = 0; i < children.length; i += 1){
            if(children[i] && children[i].isVisible() && this.checkRoleAndWall(role, children[i])){
                return true;
            }
        }
    },
    // 检测与墙的碰撞
    checkRoleAndWall: function(role, node){
        var dir = role.dir;
        var roleRect = role.boxRect;

        if(dir === ROLE_STATE.LEFT&& node.gameRect.x <= roleRect.x && Math.abs(node.gameRect.y - roleRect.y) <= 300  && roleRect.x - node.gameRect.x <= roleRect.width + node.gameRect.width){
            if(node.getParent().getParent().getParent().isVisible() && this.checkRoleHit(role, node)){
                SoundMgr.playEffect(soundRes.sound_hit_wall);
                this.moveLift(node);
                role.setRoleDir(ROLE_STATE.RIGHT);
                return true;
            }
        }else if(dir === ROLE_STATE.RIGHT&& node.gameRect.x >= roleRect.x  && Math.abs(node.gameRect.y - role.y) <= 300 && node.gameRect.x - role.x <= roleRect.width + node.gameRect.width){
            if(node.getParent().getParent().getParent().isVisible() && this.checkRoleHit(role, node)){
                SoundMgr.playEffect(soundRes.sound_hit_wall);
                this.moveLift(node);
                role.setRoleDir(ROLE_STATE.LEFT);
                return true;
            }
        }
        return false;
    },
    // 移动电梯
    moveLift: function(floor){
        var map = this.getCurrentMap(floor);
        if(map.id < 4000){ return; }
        var value = Utils.getNumberFromString(floor.getName());
        if(value > 2){
            map.closeWarning(floor);
            return;
        }
        if(map.id > 4000){  // 表示升降梯模式
            var index = 3;
            if(map.id === 4022 || map.id === 4032 || map.id === 4033){
                index = 2;
            }
            var num = Utils.getRandomNum(index);
            var y = floor.y + num * 100;
            var otherMap = map.getOtherPulley(floor);
            if(Math.abs(y - otherMap.y) <= 20){  // 同层有滑梯
                y += 100;
            }

            if(y < otherMap.y){ // 高于另一个
                y = otherMap.y + 100;
            }
            y += (this.getNextIndex(floor, y)) * 100;
            if(y >= 850){ // 不能超过最高层
                y = 850
            }

            if(floor.y >= 850){
                SoundMgr.playEffect(soundRes.sound_hit_hook);
            } else {
                SoundMgr.playEffect(soundRes.sound_hit_hook);
                SoundMgr.playEffect(soundRes.sound_hook_up);
            }

            var time = (y-floor.y)/100 * 0.03;
            floor.runAction(cc.moveTo(time, cc.p(floor.x, y)));
            map.ropeChane(floor,y,time);
        }
    },
    // 获取下一次的位置
    getNextIndex: function(target, height){
        var index = Math.ceil(height/100);
        var map = this.getCurrentMap(target);
        var node = map.getFloorsNode();
        var children =  node.getChildren();

        for(var i = 0; i <= children.length; i += 1){
            if(index + i >= children.length){
                return 0;
            }
            var child = node.getChildByName("floor_" + (index + i));
            if(child.x < 282 && child.x > 262){
                return i;
            } else if((target.x < 262 && child.x < 262) || (target.x > 282 && child.x > 282)){
                return i;
            }
        }
        return 0;
    },
    // 检测2个精灵的碰撞
    checkRoleHit: function(role, sp){
        var roleRect = role.getRoleBoundingBox();
        if((role.getRoleDir() === ROLE_STATE.LEFT && roleRect.x > sp.gameRect.x &&roleRect.x - sp.gameRect.x > sp.gameRect.width)
        || (role.getRoleDir() === ROLE_STATE.RIGHT && roleRect.x < sp.gameRect.x && sp.gameRect.x - roleRect.x > roleRect.width)){
            return false;
        }
        if(cc.rectIntersectsRect(roleRect, sp.gameRect)){
            return true;
        }
        return false;
    },
    // 检测脚下是否有底板
    checkFloorUnderRole: function(role, floors){
        if(!role || !floors){
            return false;
        }
        for(var i = 0; i < floors.length; i += 1){
            if( floors[i].isVisible() && Math.abs(floors[i].gameRect.y - role.y) <= 400 && this.checkRoleHit(role, floors[i])){
                // role.y = floors[i].gameRect.y + floors[i].gameRect.height + role.getRoleBoundingBox().height - 5;  // 校正角色位置
                role.y = floors[i].gameRect.y + 12;  // 校正角色位置

                this.curFloor = floors[i];

                if(!this.maxFloor || this.curFloor.getTag() > this.maxFloor.getTag()){  // 获取最大的地板
                    this.maxFloor = this.curFloor;
                }
                if(this.curFloor.getTag() > Global.curLevel){
                    Global.curLevel = this.curFloor.getTag();
                    this.nextFloor = this.curFloor;
                    Event.emit(EVENT_ID.JUMP);
                }
                return true;
            }
        }
        return false;
    },
    // 检测是否会死亡
    checkRoleIsDie: function(role, floors){
        if(!role || !floors){
            return false;
        }
        for(var i = 0; i < floors.length; i += 1){
            if(floors[i].isVisible() && Math.abs(floors[i].gameRect.y - role.y) <= 300 && this.isWillHitFloor(role, floors[i])){
                this.nextFloor = floors[i];
                if(this.maxFloor && this.nextFloor.getTag() > this.maxFloor.getTag() && this.nextFloor.getTag() !== GameLogic.curLevel){
                    Global.curLevel = this.nextFloor.getTag();
                    Event.emit(EVENT_ID.JUMP);
                } else if (this.maxFloor !== null && this.nextFloor.getTag() <= this.maxFloor.getTag()
                    && this.nextFloor.y <= this.maxFloor.y && this.maxFloor.y - this.nextFloor.y > 50){
                    this.commands = [];
                 }
                return false;  // 不会死亡
            }
        }
        return true;   // 死亡
    },
    checkRoleWillHitWall: function(role, walls){
          for(var i = 0; i < walls.length; i += 1){
              if(walls[i].isVisible()){
                  var bool = this.isWillHitWall(role, walls[i]);
                  if(bool){
                      return true;
                  }
              }
          }
          return false;
    },
    // 是否即将撞到墙
    isWillHitWall: function(role, wall){
        var roleRect = role.getRoleBoundingBox();
        var wallRect = wall.gameRect;
        if(role.getRoleDir() === ROLE_STATE.LEFT && roleRect.x > wallRect.x + wallRect.width
            && roleRect.y > wallRect.y && roleRect.y < wallRect.y + wallRect.height - 10){
            return true;
        } else if(role.getRoleDir() === ROLE_STATE.RIGHT && roleRect.x < wallRect.x
            && roleRect.y > wallRect.y && roleRect.y < wallRect.y + wallRect.height - 10){
            return true;
        }
        return false;
    },
    // 是否即将相撞地板
    isWillHitFloor: function(role, floor){
        if(floor.gameRect.y > role.y || Math.abs(role.y - floor.gameRect.y) > 280){
            return false;
        }
        var h = role.y - floor.gameRect.y - floor.gameRect.height;
        var t = Math.sqrt(2 * h / role.data.downG); // gt^2 = 2h;
        var s = t * role.data.speedX;                // vt = s;

        var dir = role.getRoleDir();

        if(dir === ROLE_STATE.LEFT && role.x - s >= floor.gameRect.x - role.getRoleBoundingBox().width
            && role.x - s <= floor.gameRect.x + floor.gameRect.width){
            return true;  // 相撞了
        }else if(dir === ROLE_STATE.RIGHT && role.x + s <= floor.gameRect.x + floor.gameRect.width
            && role.x + s >= floor.gameRect.x  - role.getRoleBoundingBox().width){
            return true;
        }else{
            return false;  // 不撞
        }
    },
    // 获取地图
    getFloorMap: function(floor){
        if(floor){
            return floor.getParent().getParent().getParent();
        }
        return null;
    },
    // 是否可以移动地图
    isCanMoveMap: function(){
        if(this.curFloor && this.nextFloor){
            return this.curFloor.getTag() >= Global.curLevel;
        }else{
            return false;
        }
    },
    // 重置数据
    reset: function(){
        this.nextFloor = null;
        this.curFloor = null;
        this.maxFloor = null;
        this.commands = [];
        Global.curLevel = 1;
        Global.pause = false;
        Global.curGold = 0;
        Global.rePlayCount=1;
    },
    // 增加命令
    pushCommand: function(){
       /* if(this.getCurrentMap(this.curFloor).type === MAP_TYPE.V && this.commands.length < 1){
            this.commands.push(1);
        } else*/ if(/*this.getCurrentMap(this.curFloor).type === MAP_TYPE.H &&*/ this.commands.length <= 1){
            this.commands.push(1);
        }
    },
    // 删除命令
    popCommand: function(){
        return this.commands.pop();
    },
    time: 0,
    // 控制角色状态
    controlRole: function(role){
        if(this.commands.length >= 1 && ((role.getRoleState() === ROLE_STATE.DOWN && role.isMaxHeight)
        || (role.getRoleState() === ROLE_STATE.RUN))){

            if(this.nextFloor && Global.curLevel !== 1){
                this.curFloor = this.nextFloor;
            }
            this.popCommand();
            var pos = role.getParent().convertToWorldSpace(role.getPosition());
            if(pos.y > 407){
                return true;
            }
            role.initData();
            role.setRoleState(ROLE_STATE.UP);

            if(this.getCurrentMap(this.curFloor).type === MAP_TYPE.V){
                this.commands = [];
            }
            return true;
        }
        return false;
    },
    // 当前所在的地图
    getCurrentMap: function(floor){
        if(floor){
            var map = floor.getParent().getParent().getParent();
            return map
        }
        return null;
    },
    // 检测金币碰撞
    checkGold: function(role, golds){
        for(var i = 0; i < golds.length; i += 1){
            var roleRect = role.getRoleBoundingBox();
            var goldRect = golds[i].gameRect;
            if(Math.abs(roleRect.y - goldRect.y) > 100 || (roleRect.x - goldRect.x) > Math.abs(goldRect.width/2 + roleRect.width/2)){
                continue;
            }
            if(golds[i] && golds[i].isVisible() && this.checkRoleHit(role, golds[i])){
                SoundMgr.playEffect(soundRes.sound_eat_gold);
                // engine.gameSound.playEffect(soundRes.sound_eat_gold);
                Global.curGold += 1;
                gameUserData.gold += 1;
                Event.emit(EVENT_ID.REFRESH_GOLD,1);
                golds[i].setVisible(false);
            }
        }

    },
    // 地图集合
    loadMap: function(layer){
        for(var i in Global.mapID){
            var maps = Global.mapID[i];
            for(var j = 0; j < maps.length; j += 1){
                var array = maps[j];
                for(var k = 0; k < array.length; k += 1){
                    var map = new MapObstacle(array[k]);
                    map.setVisible(false);
                    layer.addChild(map);
                    this.mapNodes.push(map);
                }
            }
        }
    },
    // 获取 新图的id
    getMapId: function(){
        this.randNum++;
        if(this.randNum >= 2000){
            this.randNum  = 0;
            return 1011;
        }
        var type = 1;
        var id = 1;
        if(this.beforeType === 1){
            type = Utils.getRandomNum(4); // 获取类型
        } else if(this.beforeType === 0){
            type = 1;
        }
        if(Global.curLevel > Global.hards[0] && Global.curLevel < Global.hards[1]){
            id = 2;
        } else if(Global.curLevel >= Global.hards[1]){
            id = 3
        }
        var mapArray = Global.mapID["type"+ type][id - 1];
        var index = Utils.getRandomNum(mapArray.length) - 1;
        var mapID = mapArray[index];

        if(this.isAlreadyOn(mapID)){
            return this.getMapId();
        } else {
            this.beforeType = type;
            return mapID;
        }
    },
    // 判断地图是否已经在地图上了
    isAlreadyOn: function(id){
        for(var i = 0; i < this.mapNodes.length; i += 1){
            if(this.mapNodes[i].isVisible() && this.mapNodes[i].id === id){
                return true;
            }
        }
        return false;
    },
    // 获取地图
    getMapNodeBy: function(id){
        for(var i = 0; i < this.mapNodes.length; i += 1){
            if(this.mapNodes[i].id === id){
                return this.mapNodes[i];
            }
        }
        return null;
    },
    // 产生金币数量
    getGoldNum: function(level){
        level = level || Global.curLevel;
        if(level <= Global.goldNums[0].level){
            return Global.goldNums[0].num;
        } else if(level > Global.goldNums[0].level && level <= Global.goldNums[1].level){
            return Global.goldNums[1].num;
        } else if(level > Global.goldNums[1].level && level <= Global.goldNums[2].level){
            return Global.goldNums[2].num;
        } else if(level > Global.goldNums[2].level){
            return 4;
        }
    },
    // 是否已经拥有
    isInArray: function(array, id){
        for(var i = 0; i < array.length; i += 1){
            if(array[i] === id){
                return true;
            }
        }
        return false;
    }
};