var Utils = {
    // 是否拥有角色
    hasRole: function(id){
        for(var i = 0; i < gameUserData.roles.length; i += 1){
            if(id === gameUserData.roles[i]){
                return true;
            }
        }
        return false;
    },
    // 获取角色动画
    getRoleAction: function(id){
        switch(id){
            case 1001:
                return AnimationMgr.getPrincessRoleRunAction();
            case 1002:
                return AnimationMgr.getNinjaRoleRunAction();
            case 1003:
                return AnimationMgr.getCaptainRoleRunAction();
            case 1004:
                return AnimationMgr.getWorkerRoleRunAction();
            case 1005:
                return AnimationMgr.getSkeletonRoleRunAction();
            case 1006:
                return AnimationMgr.getRobotRoleRunAction();
            case 1007:
                return AnimationMgr.getSpacemanRoleRunAction();
            default:
                cc.log("getRoleAction is out of id");
                break;
        }
        return AnimationMgr.getCaptainRoleRunAction();
    },
    // 获取数据
    getDataByID: function(data, id){
        for(var i = 0; i < data.length; i += 1){
            if(id === data[i].id){
                return data[i];
            }
        }
        return false;
    },
    // 换图片
    changeTexture: function(target, imgName){
        if(!target || !imgName){ return ;}

        var spriteFrame = cc.spriteFrameCache.getSpriteFrame(imgName);
        if (spriteFrame){
            target.initWithSpriteFrame(spriteFrame);
        }
    },
    // 数组里随机出一个
    randomFromArray: function(array){
        var index = Utils.getRandomNum(array.length) - 1;
        return array[index];
    },
    // 解析json
    parseNodeToUI: function(data,num){
        var children = data.Children;
        var root = new cc.Node();
        if(!children){ return root;}

        for(var i = 0; i < children.length; i += 1){
            var child = this.parseData(children[i],num);
            if(child){
                root[children[i].Name] = child;
                root.addChild(child);
            }
        }
        return root;
    },
    // 解析数据
    parseData: function(data,num){
        var node = null;
        switch(data.ctype){
            case "SpriteObjectData":
                node = this.parseSprite(data,num);
                break;
            case "TextBMFontObjectData":
                node = this.parseFontBMP(data, num);
                break;
            case "SingleNodeObjectData":
                node = this.parseNodeToUI(data, num);
                break;
            case "TextObjectData":
                node = this.parseLabel(data);
                break;
        }
        return node;
    },
    // 解析lable
    parseLabel: function(data){
        var text = new cc.LabelTTF(data.LabelText);
        text.attr({x: data.Position.X,
            y: data.Position.Y,
            tag:data.Tag,
            anchorX: data.AnchorPoint.ScaleX||0,
            anchorY: data.AnchorPoint.ScaleY,
            scaleX: data.Scale.ScaleX,
            scaleY: data.Scale.ScaleY});
        text.setName(data.Name);
        text.setFontSize(data.FontSize);
        if(data.HorizontalAlignmentType && data.HorizontalAlignmentType === "HT_Center"){
            text.setHorizontalAlignment(cc.TEXT_ALIGNMENT_CENTER);
        }
        if(data.VerticalAlignmentType && data.VerticalAlignmentType === "VT_Center"){
            text.setVerticalAlignment(cc.VERTICAL_TEXT_ALIGNMENT_CENTER);
        }
        if(data.IsCustomSize){
            text.setDimensions(cc.size(data.Size.X, data.Size.Y));
        }
        return text;
    },
    // 解析图片
    parseSprite: function(data, num){
        var sp = null;
        if(data.FileData.Type === "Normal"){
            sp = new cc.Sprite("res/"+data.FileData.Path);
            // sp = new ccui.ImageView("res/"+data.FileData.Path);
        } else {
            sp = new cc.Sprite("#"+ data.FileData.Path);
            // sp = new ccui.ImageView("#"+ data.FileData.Path);
        }

        sp.attr({x: data.Position.X,
            y: data.Position.Y,
            tag:data.Tag + num * 10,
            anchorX: data.AnchorPoint.ScaleX,
            anchorY: data.AnchorPoint.ScaleY,
            scaleX: data.Scale.ScaleX,
            scaleY: data.Scale.ScaleY});
         if(data.RotationSkewX){
            sp.setRotationX(data.RotationSkewX);
        }
        if(data.RotationSkewY){
            sp.setRotationY(data.RotationSkewY)
        }
        sp.setName(data.Name);
        // sp.addChild(this.drawNode(sp.getBoundingBox()));
        return sp;
    },
    // 解析文字
    parseFontBMP: function(data, num){
        var str = data.LabelText;
        var path = "res/" + data.LabelBMFontFile_CNB.Path;
        var text = new cc.LabelBMFont((num + 1)*10, path);
        text.attr({x: data.Position.X,
            y: data.Position.Y,
            tag:data.Tag,
            anchorX: data.AnchorPoint.ScaleX||0,
            anchorY: data.AnchorPoint.ScaleY,
            scaleX: data.Scale.ScaleX,
            scaleY: data.Scale.ScaleY});
        text.setName(data.Name);
        return text;
    },
    drawNode: function(rect){
        // var rect = sp.getBoundingBox();
        var node = new cc.DrawNode();
        // node.setAnchorPoint(sp.getAnchorPoint());
        node.drawRect(cc.p(rect.x, rect.y), cc.p(rect.width + rect.x, rect.height + rect.y), cc.color(0,0,0,0), 2, cc.color(255,0,0,255));
        return node;
    },
    // 获取初始化速度
    getOriginalSpeed: function(g, h){
        var speed = Math.sqrt(2 * g * h);
        return speed;
    },
    // 获取上升到最大高度的时间
    getMaxHeightTime: function(g,h){
        var time = Math.sqrt(2*h/g);
        return time;
    },
    // 获取地图的json
    getMapJsonByID: function(id){
        return "res/map/map_" + id + ".json";
    },
    // 获取背景的json
    getBgJsonByID: function(id){
        switch(id){
            case 2001:
                return res.res_game_bg_1_json;
            case 2002:
                return res.res_game_bg_japan_json;
            case 2003:
                return res.res_game_bg_egypt_json;
            case 2004:
                return res.res_game_bg_russia_json;
            case 2005:
                return res.res_game_bg_venice_json;
            default:
                cc.log(" function getBgJsonByID() is out of ID !!!");
                break;
        }
    },
    // 取出整数
    getNumberFromString: function(str){
        var regEx = /\d+/g;
        return Number(regEx.exec(str));
    },
    // 生成随机数
    getRandomNum: function(seed){
        seed = seed || 100;
        var num = Math.floor(Math.random()*seed)  + 1; // 生成 1- seed 的数值
        return num;
    },
    // 从数组中移除数据
    removeDataFromArray: function(array, id){
        for(var i = 0; i < array.length; i += 1){
            if(array[i] === id){
                array.splice(i, 1);
            }
        }
    }
};
